# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command(
    "vm host group show",
)
class Show(AAZCommand):
    """Get the details of a dedicated host group.

    :example: Get the details of a dedicated host group.
        az vm host group show --name MyDedicatedHostGroup --resource-group MyResourceGroup
    """

    _aaz_info = {
        "version": "2024-11-01",
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.compute/hostgroups/{}", "2024-11-01"],
        ]
    }

    def _handler(self, command_args):
        super()._handler(command_args)
        self._execute_operations()
        return self._output()

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.host_group_name = AAZStrArg(
            options=["-n", "--name", "--host-group-name"],
            help="Name of the Dedicated Host Group.",
            required=True,
            id_part="name",
        )
        _args_schema.resource_group = AAZResourceGroupNameArg(
            required=True,
        )
        _args_schema.expand = AAZStrArg(
            options=["--expand"],
            help="The expand expression to apply on the operation. 'InstanceView' will retrieve the list of instance views of the dedicated hosts under the dedicated host group. 'UserData' is not supported for dedicated host group.",
            enum={"instanceView": "instanceView", "resiliencyView": "resiliencyView", "userData": "userData"},
        )
        return cls._args_schema

    def _execute_operations(self):
        self.pre_operations()
        self.DedicatedHostGroupsGet(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result

    class DedicatedHostGroupsGet(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [200]:
                return self.on_200(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Compute/hostGroups/{hostGroupName}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "GET"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "hostGroupName", self.ctx.args.host_group_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "$expand", self.ctx.args.expand,
                ),
                **self.serialize_query_param(
                    "api-version", "2024-11-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        def on_200(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType()

            _schema_on_200 = cls._schema_on_200
            _schema_on_200.id = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200.location = AAZStrType(
                flags={"required": True},
            )
            _schema_on_200.name = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200.properties = AAZObjectType(
                flags={"client_flatten": True},
            )
            _schema_on_200.system_data = AAZObjectType(
                serialized_name="systemData",
                flags={"read_only": True},
            )
            _schema_on_200.tags = AAZDictType()
            _schema_on_200.type = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200.zones = AAZListType()

            properties = cls._schema_on_200.properties
            properties.additional_capabilities = AAZObjectType(
                serialized_name="additionalCapabilities",
            )
            properties.hosts = AAZListType(
                flags={"read_only": True},
            )
            properties.instance_view = AAZObjectType(
                serialized_name="instanceView",
                flags={"read_only": True},
            )
            properties.platform_fault_domain_count = AAZIntType(
                serialized_name="platformFaultDomainCount",
                flags={"required": True},
            )
            properties.support_automatic_placement = AAZBoolType(
                serialized_name="supportAutomaticPlacement",
            )

            additional_capabilities = cls._schema_on_200.properties.additional_capabilities
            additional_capabilities.ultra_ssd_enabled = AAZBoolType(
                serialized_name="ultraSSDEnabled",
            )

            hosts = cls._schema_on_200.properties.hosts
            hosts.Element = AAZObjectType()

            _element = cls._schema_on_200.properties.hosts.Element
            _element.id = AAZStrType(
                flags={"read_only": True},
            )

            instance_view = cls._schema_on_200.properties.instance_view
            instance_view.hosts = AAZListType()

            hosts = cls._schema_on_200.properties.instance_view.hosts
            hosts.Element = AAZObjectType()

            _element = cls._schema_on_200.properties.instance_view.hosts.Element
            _element.asset_id = AAZStrType(
                serialized_name="assetId",
                flags={"read_only": True},
            )
            _element.available_capacity = AAZObjectType(
                serialized_name="availableCapacity",
            )
            _element.name = AAZStrType(
                flags={"read_only": True},
            )
            _element.statuses = AAZListType()

            available_capacity = cls._schema_on_200.properties.instance_view.hosts.Element.available_capacity
            available_capacity.allocatable_v_ms = AAZListType(
                serialized_name="allocatableVMs",
            )

            allocatable_v_ms = cls._schema_on_200.properties.instance_view.hosts.Element.available_capacity.allocatable_v_ms
            allocatable_v_ms.Element = AAZObjectType()

            _element = cls._schema_on_200.properties.instance_view.hosts.Element.available_capacity.allocatable_v_ms.Element
            _element.count = AAZFloatType()
            _element.vm_size = AAZStrType(
                serialized_name="vmSize",
            )

            statuses = cls._schema_on_200.properties.instance_view.hosts.Element.statuses
            statuses.Element = AAZObjectType()

            _element = cls._schema_on_200.properties.instance_view.hosts.Element.statuses.Element
            _element.code = AAZStrType()
            _element.display_status = AAZStrType(
                serialized_name="displayStatus",
            )
            _element.level = AAZStrType()
            _element.message = AAZStrType()
            _element.time = AAZStrType()

            system_data = cls._schema_on_200.system_data
            system_data.created_at = AAZStrType(
                serialized_name="createdAt",
            )
            system_data.created_by = AAZStrType(
                serialized_name="createdBy",
            )
            system_data.created_by_type = AAZStrType(
                serialized_name="createdByType",
            )
            system_data.last_modified_at = AAZStrType(
                serialized_name="lastModifiedAt",
            )
            system_data.last_modified_by = AAZStrType(
                serialized_name="lastModifiedBy",
            )
            system_data.last_modified_by_type = AAZStrType(
                serialized_name="lastModifiedByType",
            )

            tags = cls._schema_on_200.tags
            tags.Element = AAZStrType()

            zones = cls._schema_on_200.zones
            zones.Element = AAZStrType()

            return cls._schema_on_200


class _ShowHelper:
    """Helper class for Show"""


__all__ = ["Show"]



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from typing import Any, List, Optional, TYPE_CHECKING

from ... import _serialization

if TYPE_CHECKING:
    from .. import models as _models


class ResourceSku(_serialization.Model):
    """Describes an available Compute SKU.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar resource_type: The type of resource the SKU applies to.
    :vartype resource_type: str
    :ivar name: The name of SKU.
    :vartype name: str
    :ivar tier: Specifies the tier of virtual machines in a scale set.\\ :code:`<br />`\\
     :code:`<br />` Possible Values:\\ :code:`<br />`\\ :code:`<br />` **Standard**\\ :code:`<br
     />`\\ :code:`<br />` **Basic**.
    :vartype tier: str
    :ivar size: The Size of the SKU.
    :vartype size: str
    :ivar family: The Family of this particular SKU.
    :vartype family: str
    :ivar kind: The Kind of resources that are supported in this SKU.
    :vartype kind: str
    :ivar capacity: Specifies the number of virtual machines in the scale set.
    :vartype capacity: ~azure.mgmt.compute.v2019_04_01.models.ResourceSkuCapacity
    :ivar locations: The set of locations that the SKU is available.
    :vartype locations: list[str]
    :ivar location_info: A list of locations and availability zones in those locations where the
     SKU is available.
    :vartype location_info: list[~azure.mgmt.compute.v2019_04_01.models.ResourceSkuLocationInfo]
    :ivar api_versions: The api versions that support this SKU.
    :vartype api_versions: list[str]
    :ivar costs: Metadata for retrieving price info.
    :vartype costs: list[~azure.mgmt.compute.v2019_04_01.models.ResourceSkuCosts]
    :ivar capabilities: A name value pair to describe the capability.
    :vartype capabilities: list[~azure.mgmt.compute.v2019_04_01.models.ResourceSkuCapabilities]
    :ivar restrictions: The restrictions because of which SKU cannot be used. This is empty if
     there are no restrictions.
    :vartype restrictions: list[~azure.mgmt.compute.v2019_04_01.models.ResourceSkuRestrictions]
    """

    _validation = {
        "resource_type": {"readonly": True},
        "name": {"readonly": True},
        "tier": {"readonly": True},
        "size": {"readonly": True},
        "family": {"readonly": True},
        "kind": {"readonly": True},
        "capacity": {"readonly": True},
        "locations": {"readonly": True},
        "location_info": {"readonly": True},
        "api_versions": {"readonly": True},
        "costs": {"readonly": True},
        "capabilities": {"readonly": True},
        "restrictions": {"readonly": True},
    }

    _attribute_map = {
        "resource_type": {"key": "resourceType", "type": "str"},
        "name": {"key": "name", "type": "str"},
        "tier": {"key": "tier", "type": "str"},
        "size": {"key": "size", "type": "str"},
        "family": {"key": "family", "type": "str"},
        "kind": {"key": "kind", "type": "str"},
        "capacity": {"key": "capacity", "type": "ResourceSkuCapacity"},
        "locations": {"key": "locations", "type": "[str]"},
        "location_info": {"key": "locationInfo", "type": "[ResourceSkuLocationInfo]"},
        "api_versions": {"key": "apiVersions", "type": "[str]"},
        "costs": {"key": "costs", "type": "[ResourceSkuCosts]"},
        "capabilities": {"key": "capabilities", "type": "[ResourceSkuCapabilities]"},
        "restrictions": {"key": "restrictions", "type": "[ResourceSkuRestrictions]"},
    }

    def __init__(self, **kwargs: Any) -> None:
        """ """
        super().__init__(**kwargs)
        self.resource_type = None
        self.name = None
        self.tier = None
        self.size = None
        self.family = None
        self.kind = None
        self.capacity = None
        self.locations = None
        self.location_info = None
        self.api_versions = None
        self.costs = None
        self.capabilities = None
        self.restrictions = None


class ResourceSkuCapabilities(_serialization.Model):
    """Describes The SKU capabilities object.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar name: An invariant to describe the feature.
    :vartype name: str
    :ivar value: An invariant if the feature is measured by quantity.
    :vartype value: str
    """

    _validation = {
        "name": {"readonly": True},
        "value": {"readonly": True},
    }

    _attribute_map = {
        "name": {"key": "name", "type": "str"},
        "value": {"key": "value", "type": "str"},
    }

    def __init__(self, **kwargs: Any) -> None:
        """ """
        super().__init__(**kwargs)
        self.name = None
        self.value = None


class ResourceSkuCapacity(_serialization.Model):
    """Describes scaling information of a SKU.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar minimum: The minimum capacity.
    :vartype minimum: int
    :ivar maximum: The maximum capacity that can be set.
    :vartype maximum: int
    :ivar default: The default capacity.
    :vartype default: int
    :ivar scale_type: The scale type applicable to the sku. Known values are: "Automatic",
     "Manual", and "None".
    :vartype scale_type: str or ~azure.mgmt.compute.v2019_04_01.models.ResourceSkuCapacityScaleType
    """

    _validation = {
        "minimum": {"readonly": True},
        "maximum": {"readonly": True},
        "default": {"readonly": True},
        "scale_type": {"readonly": True},
    }

    _attribute_map = {
        "minimum": {"key": "minimum", "type": "int"},
        "maximum": {"key": "maximum", "type": "int"},
        "default": {"key": "default", "type": "int"},
        "scale_type": {"key": "scaleType", "type": "str"},
    }

    def __init__(self, **kwargs: Any) -> None:
        """ """
        super().__init__(**kwargs)
        self.minimum = None
        self.maximum = None
        self.default = None
        self.scale_type = None


class ResourceSkuCosts(_serialization.Model):
    """Describes metadata for retrieving price info.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar meter_id: Used for querying price from commerce.
    :vartype meter_id: str
    :ivar quantity: The multiplier is needed to extend the base metered cost.
    :vartype quantity: int
    :ivar extended_unit: An invariant to show the extended unit.
    :vartype extended_unit: str
    """

    _validation = {
        "meter_id": {"readonly": True},
        "quantity": {"readonly": True},
        "extended_unit": {"readonly": True},
    }

    _attribute_map = {
        "meter_id": {"key": "meterID", "type": "str"},
        "quantity": {"key": "quantity", "type": "int"},
        "extended_unit": {"key": "extendedUnit", "type": "str"},
    }

    def __init__(self, **kwargs: Any) -> None:
        """ """
        super().__init__(**kwargs)
        self.meter_id = None
        self.quantity = None
        self.extended_unit = None


class ResourceSkuLocationInfo(_serialization.Model):
    """ResourceSkuLocationInfo.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar location: Location of the SKU.
    :vartype location: str
    :ivar zones: List of availability zones where the SKU is supported.
    :vartype zones: list[str]
    :ivar zone_details: Details of capabilities available to a SKU in specific zones.
    :vartype zone_details: list[~azure.mgmt.compute.v2019_04_01.models.ResourceSkuZoneDetails]
    """

    _validation = {
        "location": {"readonly": True},
        "zones": {"readonly": True},
        "zone_details": {"readonly": True},
    }

    _attribute_map = {
        "location": {"key": "location", "type": "str"},
        "zones": {"key": "zones", "type": "[str]"},
        "zone_details": {"key": "zoneDetails", "type": "[ResourceSkuZoneDetails]"},
    }

    def __init__(self, **kwargs: Any) -> None:
        """ """
        super().__init__(**kwargs)
        self.location = None
        self.zones = None
        self.zone_details = None


class ResourceSkuRestrictionInfo(_serialization.Model):
    """ResourceSkuRestrictionInfo.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar locations: Locations where the SKU is restricted.
    :vartype locations: list[str]
    :ivar zones: List of availability zones where the SKU is restricted.
    :vartype zones: list[str]
    """

    _validation = {
        "locations": {"readonly": True},
        "zones": {"readonly": True},
    }

    _attribute_map = {
        "locations": {"key": "locations", "type": "[str]"},
        "zones": {"key": "zones", "type": "[str]"},
    }

    def __init__(self, **kwargs: Any) -> None:
        """ """
        super().__init__(**kwargs)
        self.locations = None
        self.zones = None


class ResourceSkuRestrictions(_serialization.Model):
    """Describes scaling information of a SKU.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar type: The type of restrictions. Known values are: "Location" and "Zone".
    :vartype type: str or ~azure.mgmt.compute.v2019_04_01.models.ResourceSkuRestrictionsType
    :ivar values: The value of restrictions. If the restriction type is set to location. This would
     be different locations where the SKU is restricted.
    :vartype values: list[str]
    :ivar restriction_info: The information about the restriction where the SKU cannot be used.
    :vartype restriction_info: ~azure.mgmt.compute.v2019_04_01.models.ResourceSkuRestrictionInfo
    :ivar reason_code: The reason for restriction. Known values are: "QuotaId" and
     "NotAvailableForSubscription".
    :vartype reason_code: str or
     ~azure.mgmt.compute.v2019_04_01.models.ResourceSkuRestrictionsReasonCode
    """

    _validation = {
        "type": {"readonly": True},
        "values": {"readonly": True},
        "restriction_info": {"readonly": True},
        "reason_code": {"readonly": True},
    }

    _attribute_map = {
        "type": {"key": "type", "type": "str"},
        "values": {"key": "values", "type": "[str]"},
        "restriction_info": {"key": "restrictionInfo", "type": "ResourceSkuRestrictionInfo"},
        "reason_code": {"key": "reasonCode", "type": "str"},
    }

    def __init__(self, **kwargs: Any) -> None:
        """ """
        super().__init__(**kwargs)
        self.type = None
        self.values = None
        self.restriction_info = None
        self.reason_code = None


class ResourceSkusResult(_serialization.Model):
    """The List Resource Skus operation response.

    All required parameters must be populated in order to send to server.

    :ivar value: The list of skus available for the subscription. Required.
    :vartype value: list[~azure.mgmt.compute.v2019_04_01.models.ResourceSku]
    :ivar next_link: The URI to fetch the next page of Resource Skus. Call ListNext() with this URI
     to fetch the next page of Resource Skus.
    :vartype next_link: str
    """

    _validation = {
        "value": {"required": True},
    }

    _attribute_map = {
        "value": {"key": "value", "type": "[ResourceSku]"},
        "next_link": {"key": "nextLink", "type": "str"},
    }

    def __init__(self, *, value: List["_models.ResourceSku"], next_link: Optional[str] = None, **kwargs: Any) -> None:
        """
        :keyword value: The list of skus available for the subscription. Required.
        :paramtype value: list[~azure.mgmt.compute.v2019_04_01.models.ResourceSku]
        :keyword next_link: The URI to fetch the next page of Resource Skus. Call ListNext() with this
         URI to fetch the next page of Resource Skus.
        :paramtype next_link: str
        """
        super().__init__(**kwargs)
        self.value = value
        self.next_link = next_link


class ResourceSkuZoneDetails(_serialization.Model):
    """Describes The zonal capabilities of a SKU.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar name: The set of zones that the SKU is available in with the specified capabilities.
    :vartype name: list[str]
    :ivar capabilities: A list of capabilities that are available for the SKU in the specified list
     of zones.
    :vartype capabilities: list[~azure.mgmt.compute.v2019_04_01.models.ResourceSkuCapabilities]
    """

    _validation = {
        "name": {"readonly": True},
        "capabilities": {"readonly": True},
    }

    _attribute_map = {
        "name": {"key": "name", "type": "[str]"},
        "capabilities": {"key": "capabilities", "type": "[ResourceSkuCapabilities]"},
    }

    def __init__(self, **kwargs: Any) -> None:
        """ """
        super().__init__(**kwargs)
        self.name = None
        self.capabilities = None



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
from enum import Enum
from typing import Dict, Mapping, Optional, Union, Sequence, TypedDict

from .._enum_meta import CaseInsensitiveEnumMeta


AttributeValue = Union[
    str,
    bool,
    int,
    float,
    Sequence[str],
    Sequence[bool],
    Sequence[int],
    Sequence[float],
]
Attributes = Mapping[str, AttributeValue]


class SpanKind(Enum, metaclass=CaseInsensitiveEnumMeta):
    """Describes the role or kind of a span within a distributed trace.

    This helps to categorize spans based on their relationship to other spans and the type
    of operation they represent.
    """

    UNSPECIFIED = 1
    """Unspecified span kind."""

    SERVER = 2
    """Indicates that the span describes an operation that handles a remote request."""

    CLIENT = 3
    """Indicates that the span describes a request to some remote service."""

    PRODUCER = 4
    """Indicates that the span describes the initiation or scheduling of a local or remote operation."""

    CONSUMER = 5
    """Indicates that the span represents the processing of an operation initiated by a producer."""

    INTERNAL = 6
    """Indicates that the span is used internally in the application."""


class Link:
    """Represents a reference from one span to another span.

    :param headers: A dictionary of the request header as key value pairs.
    :type headers: dict
    :param attributes: Any additional attributes that should be added to link
    :type attributes: dict
    """

    def __init__(self, headers: Dict[str, str], attributes: Optional[Attributes] = None) -> None:
        self.headers = headers
        self.attributes = attributes


class TracingOptions(TypedDict, total=False):
    """Options to configure tracing behavior for operations."""

    enabled: bool
    """Whether tracing is enabled for the operation. By default, if the global setting is enabled, tracing is
    enabled for all operations. This option can be used to override the global setting for a specific operation."""
    attributes: Attributes
    """Attributes to include in the spans emitted for the operation."""



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

try:
    from ._models_py3 import Creator
    from ._models_py3 import CreatorList
    from ._models_py3 import CreatorProperties
    from ._models_py3 import CreatorUpdateParameters
    from ._models_py3 import Dimension
    from ._models_py3 import ErrorAdditionalInfo
    from ._models_py3 import ErrorDetail
    from ._models_py3 import ErrorResponse
    from ._models_py3 import MapsAccount
    from ._models_py3 import MapsAccountKeys
    from ._models_py3 import MapsAccountProperties
    from ._models_py3 import MapsAccountUpdateParameters
    from ._models_py3 import MapsAccounts
    from ._models_py3 import MapsKeySpecification
    from ._models_py3 import MapsOperations
    from ._models_py3 import MetricSpecification
    from ._models_py3 import OperationDetail
    from ._models_py3 import OperationDisplay
    from ._models_py3 import Resource
    from ._models_py3 import ServiceSpecification
    from ._models_py3 import Sku
    from ._models_py3 import SystemData
    from ._models_py3 import TrackedResource
except (SyntaxError, ImportError):
    from ._models import Creator  # type: ignore
    from ._models import CreatorList  # type: ignore
    from ._models import CreatorProperties  # type: ignore
    from ._models import CreatorUpdateParameters  # type: ignore
    from ._models import Dimension  # type: ignore
    from ._models import ErrorAdditionalInfo  # type: ignore
    from ._models import ErrorDetail  # type: ignore
    from ._models import ErrorResponse  # type: ignore
    from ._models import MapsAccount  # type: ignore
    from ._models import MapsAccountKeys  # type: ignore
    from ._models import MapsAccountProperties  # type: ignore
    from ._models import MapsAccountUpdateParameters  # type: ignore
    from ._models import MapsAccounts  # type: ignore
    from ._models import MapsKeySpecification  # type: ignore
    from ._models import MapsOperations  # type: ignore
    from ._models import MetricSpecification  # type: ignore
    from ._models import OperationDetail  # type: ignore
    from ._models import OperationDisplay  # type: ignore
    from ._models import Resource  # type: ignore
    from ._models import ServiceSpecification  # type: ignore
    from ._models import Sku  # type: ignore
    from ._models import SystemData  # type: ignore
    from ._models import TrackedResource  # type: ignore

from ._azure_maps_management_client_enums import (
    CreatedByType,
    KeyType,
    Kind,
    Name,
)

__all__ = [
    'Creator',
    'CreatorList',
    'CreatorProperties',
    'CreatorUpdateParameters',
    'Dimension',
    'ErrorAdditionalInfo',
    'ErrorDetail',
    'ErrorResponse',
    'MapsAccount',
    'MapsAccountKeys',
    'MapsAccountProperties',
    'MapsAccountUpdateParameters',
    'MapsAccounts',
    'MapsKeySpecification',
    'MapsOperations',
    'MetricSpecification',
    'OperationDetail',
    'OperationDisplay',
    'Resource',
    'ServiceSpecification',
    'Sku',
    'SystemData',
    'TrackedResource',
    'CreatedByType',
    'KeyType',
    'Kind',
    'Name',
]



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from ._models import AffinityInfo
from ._models import AuthenticationTokenSettings
from ._models import AutoScaleRun
from ._models import AutoScaleRunError
from ._models import AutoUserSpecification
from ._models import AutomaticOsUpgradePolicy
from ._models import AzureBlobFileSystemConfiguration
from ._models import AzureFileShareConfiguration
from ._models import BatchAccountListSupportedImagesResult
from ._models import BatchApplication
from ._models import BatchApplicationListResult
from ._models import BatchApplicationPackageReference
from ._models import BatchAutoPoolSpecification
from ._models import BatchError
from ._models import BatchErrorDetail
from ._models import BatchErrorMessage
from ._models import BatchJob
from ._models import BatchJobConstraints
from ._models import BatchJobCreateContent
from ._models import BatchJobDisableContent
from ._models import BatchJobExecutionInfo
from ._models import BatchJobListResult
from ._models import BatchJobManagerTask
from ._models import BatchJobNetworkConfiguration
from ._models import BatchJobPreparationAndReleaseTaskStatus
from ._models import BatchJobPreparationAndReleaseTaskStatusListResult
from ._models import BatchJobPreparationTask
from ._models import BatchJobPreparationTaskExecutionInfo
from ._models import BatchJobReleaseTask
from ._models import BatchJobReleaseTaskExecutionInfo
from ._models import BatchJobSchedule
from ._models import BatchJobScheduleConfiguration
from ._models import BatchJobScheduleCreateContent
from ._models import BatchJobScheduleExecutionInfo
from ._models import BatchJobScheduleListResult
from ._models import BatchJobScheduleStatistics
from ._models import BatchJobScheduleUpdateContent
from ._models import BatchJobSchedulingError
from ._models import BatchJobSpecification
from ._models import BatchJobStatistics
from ._models import BatchJobTerminateContent
from ._models import BatchJobUpdateContent
from ._models import BatchNode
from ._models import BatchNodeAgentInfo
from ._models import BatchNodeCounts
from ._models import BatchNodeDisableSchedulingContent
from ._models import BatchNodeEndpointConfiguration
from ._models import BatchNodeError
from ._models import BatchNodeFile
from ._models import BatchNodeFileListResult
from ._models import BatchNodeIdentityReference
from ._models import BatchNodeInfo
from ._models import BatchNodeListResult
from ._models import BatchNodePlacementConfiguration
from ._models import BatchNodeRebootContent
from ._models import BatchNodeRemoteLoginSettings
from ._models import BatchNodeRemoveContent
from ._models import BatchNodeUserCreateContent
from ._models import BatchNodeUserUpdateContent
from ._models import BatchNodeVMExtension
from ._models import BatchNodeVMExtensionListResult
from ._models import BatchPool
from ._models import BatchPoolCreateContent
from ._models import BatchPoolEnableAutoScaleContent
from ._models import BatchPoolEndpointConfiguration
from ._models import BatchPoolEvaluateAutoScaleContent
from ._models import BatchPoolIdentity
from ._models import BatchPoolInfo
from ._models import BatchPoolListResult
from ._models import BatchPoolListUsageMetricsResult
from ._models import BatchPoolNodeCounts
from ._models import BatchPoolNodeCountsListResult
from ._models import BatchPoolReplaceContent
from ._models import BatchPoolResizeContent
from ._models import BatchPoolResourceStatistics
from ._models import BatchPoolSpecification
from ._models import BatchPoolStatistics
from ._models import BatchPoolUpdateContent
from ._models import BatchPoolUsageMetrics
from ._models import BatchPoolUsageStatistics
from ._models import BatchStartTask
from ._models import BatchStartTaskInfo
from ._models import BatchSubtask
from ._models import BatchSupportedImage
from ._models import BatchTask
from ._models import BatchTaskAddCollectionResult
from ._models import BatchTaskAddResult
from ._models import BatchTaskConstraints
from ._models import BatchTaskContainerExecutionInfo
from ._models import BatchTaskContainerSettings
from ._models import BatchTaskCounts
from ._models import BatchTaskCountsResult
from ._models import BatchTaskCreateContent
from ._models import BatchTaskDependencies
from ._models import BatchTaskExecutionInfo
from ._models import BatchTaskFailureInfo
from ._models import BatchTaskGroup
from ._models import BatchTaskIdRange
from ._models import BatchTaskInfo
from ._models import BatchTaskListResult
from ._models import BatchTaskListSubtasksResult
from ._models import BatchTaskSchedulingPolicy
from ._models import BatchTaskSlotCounts
from ._models import BatchTaskStatistics
from ._models import CifsMountConfiguration
from ._models import ContainerConfiguration
from ._models import ContainerRegistryReference
from ._models import DataDisk
from ._models import DiffDiskSettings
from ._models import DiskEncryptionConfiguration
from ._models import EnvironmentSetting
from ._models import ExitCodeMapping
from ._models import ExitCodeRangeMapping
from ._models import ExitConditions
from ._models import ExitOptions
from ._models import FileProperties
from ._models import HttpHeader
from ._models import ImageReference
from ._models import InboundEndpoint
from ._models import InboundNatPool
from ._models import InstanceViewStatus
from ._models import LinuxUserConfiguration
from ._models import ManagedDisk
from ._models import MetadataItem
from ._models import MountConfiguration
from ._models import MultiInstanceSettings
from ._models import NameValuePair
from ._models import NetworkConfiguration
from ._models import NetworkSecurityGroupRule
from ._models import NfsMountConfiguration
from ._models import OSDisk
from ._models import OutputFile
from ._models import OutputFileBlobContainerDestination
from ._models import OutputFileDestination
from ._models import OutputFileUploadConfig
from ._models import PublicIpAddressConfiguration
from ._models import RecentBatchJob
from ._models import ResizeError
from ._models import ResourceFile
from ._models import RollingUpgradePolicy
from ._models import SecurityProfile
from ._models import ServiceArtifactReference
from ._models import UefiSettings
from ._models import UpgradePolicy
from ._models import UploadBatchServiceLogsContent
from ._models import UploadBatchServiceLogsResult
from ._models import UserAccount
from ._models import UserAssignedIdentity
from ._models import UserIdentity
from ._models import VMExtension
from ._models import VMExtensionInstanceView
from ._models import VirtualMachineConfiguration
from ._models import VirtualMachineInfo
from ._models import WindowsConfiguration
from ._models import WindowsUserConfiguration

from ._enums import AccessScope
from ._enums import AllocationState
from ._enums import AutoUserScope
from ._enums import BatchJobAction
from ._enums import BatchJobPreparationTaskState
from ._enums import BatchJobReleaseTaskState
from ._enums import BatchJobScheduleState
from ._enums import BatchJobState
from ._enums import BatchNodeCommunicationMode
from ._enums import BatchNodeDeallocationOption
from ._enums import BatchNodeDisableSchedulingOption
from ._enums import BatchNodeFillType
from ._enums import BatchNodePlacementPolicyType
from ._enums import BatchNodeRebootOption
from ._enums import BatchNodeState
from ._enums import BatchPoolIdentityType
from ._enums import BatchPoolLifetimeOption
from ._enums import BatchPoolState
from ._enums import BatchStartTaskState
from ._enums import BatchSubtaskState
from ._enums import BatchTaskAddStatus
from ._enums import BatchTaskExecutionResult
from ._enums import BatchTaskState
from ._enums import CachingType
from ._enums import ContainerType
from ._enums import ContainerWorkingDirectory
from ._enums import DependencyAction
from ._enums import DiffDiskPlacement
from ._enums import DisableBatchJobOption
from ._enums import DiskEncryptionTarget
from ._enums import DynamicVNetAssignmentScope
from ._enums import ElevationLevel
from ._enums import ErrorCategory
from ._enums import ImageVerificationType
from ._enums import InboundEndpointProtocol
from ._enums import IpAddressProvisioningType
from ._enums import LoginMode
from ._enums import NetworkSecurityGroupRuleAccess
from ._enums import OSType
from ._enums import OnAllBatchTasksComplete
from ._enums import OnBatchTaskFailure
from ._enums import OutputFileUploadCondition
from ._enums import SchedulingState
from ._enums import SecurityTypes
from ._enums import StatusLevelTypes
from ._enums import StorageAccountType
from ._enums import UpgradeMode
from ._patch import __all__ as _patch_all
from ._patch import *  # pylint: disable=unused-wildcard-import
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "AffinityInfo",
    "AuthenticationTokenSettings",
    "AutoScaleRun",
    "AutoScaleRunError",
    "AutoUserSpecification",
    "AutomaticOsUpgradePolicy",
    "AzureBlobFileSystemConfiguration",
    "AzureFileShareConfiguration",
    "BatchAccountListSupportedImagesResult",
    "BatchApplication",
    "BatchApplicationListResult",
    "BatchApplicationPackageReference",
    "BatchAutoPoolSpecification",
    "BatchError",
    "BatchErrorDetail",
    "BatchErrorMessage",
    "BatchJob",
    "BatchJobConstraints",
    "BatchJobCreateContent",
    "BatchJobDisableContent",
    "BatchJobExecutionInfo",
    "BatchJobListResult",
    "BatchJobManagerTask",
    "BatchJobNetworkConfiguration",
    "BatchJobPreparationAndReleaseTaskStatus",
    "BatchJobPreparationAndReleaseTaskStatusListResult",
    "BatchJobPreparationTask",
    "BatchJobPreparationTaskExecutionInfo",
    "BatchJobReleaseTask",
    "BatchJobReleaseTaskExecutionInfo",
    "BatchJobSchedule",
    "BatchJobScheduleConfiguration",
    "BatchJobScheduleCreateContent",
    "BatchJobScheduleExecutionInfo",
    "BatchJobScheduleListResult",
    "BatchJobScheduleStatistics",
    "BatchJobScheduleUpdateContent",
    "BatchJobSchedulingError",
    "BatchJobSpecification",
    "BatchJobStatistics",
    "BatchJobTerminateContent",
    "BatchJobUpdateContent",
    "BatchNode",
    "BatchNodeAgentInfo",
    "BatchNodeCounts",
    "BatchNodeDisableSchedulingContent",
    "BatchNodeEndpointConfiguration",
    "BatchNodeError",
    "BatchNodeFile",
    "BatchNodeFileListResult",
    "BatchNodeIdentityReference",
    "BatchNodeInfo",
    "BatchNodeListResult",
    "BatchNodePlacementConfiguration",
    "BatchNodeRebootContent",
    "BatchNodeRemoteLoginSettings",
    "BatchNodeRemoveContent",
    "BatchNodeUserCreateContent",
    "BatchNodeUserUpdateContent",
    "BatchNodeVMExtension",
    "BatchNodeVMExtensionListResult",
    "BatchPool",
    "BatchPoolCreateContent",
    "BatchPoolEnableAutoScaleContent",
    "BatchPoolEndpointConfiguration",
    "BatchPoolEvaluateAutoScaleContent",
    "BatchPoolIdentity",
    "BatchPoolInfo",
    "BatchPoolListResult",
    "BatchPoolListUsageMetricsResult",
    "BatchPoolNodeCounts",
    "BatchPoolNodeCountsListResult",
    "BatchPoolReplaceContent",
    "BatchPoolResizeContent",
    "BatchPoolResourceStatistics",
    "BatchPoolSpecification",
    "BatchPoolStatistics",
    "BatchPoolUpdateContent",
    "BatchPoolUsageMetrics",
    "BatchPoolUsageStatistics",
    "BatchStartTask",
    "BatchStartTaskInfo",
    "BatchSubtask",
    "BatchSupportedImage",
    "BatchTask",
    "BatchTaskAddCollectionResult",
    "BatchTaskAddResult",
    "BatchTaskConstraints",
    "BatchTaskContainerExecutionInfo",
    "BatchTaskContainerSettings",
    "BatchTaskCounts",
    "BatchTaskCountsResult",
    "BatchTaskCreateContent",
    "BatchTaskDependencies",
    "BatchTaskExecutionInfo",
    "BatchTaskFailureInfo",
    "BatchTaskGroup",
    "BatchTaskIdRange",
    "BatchTaskInfo",
    "BatchTaskListResult",
    "BatchTaskListSubtasksResult",
    "BatchTaskSchedulingPolicy",
    "BatchTaskSlotCounts",
    "BatchTaskStatistics",
    "CifsMountConfiguration",
    "ContainerConfiguration",
    "ContainerRegistryReference",
    "DataDisk",
    "DiffDiskSettings",
    "DiskEncryptionConfiguration",
    "EnvironmentSetting",
    "ExitCodeMapping",
    "ExitCodeRangeMapping",
    "ExitConditions",
    "ExitOptions",
    "FileProperties",
    "HttpHeader",
    "ImageReference",
    "InboundEndpoint",
    "InboundNatPool",
    "InstanceViewStatus",
    "LinuxUserConfiguration",
    "ManagedDisk",
    "MetadataItem",
    "MountConfiguration",
    "MultiInstanceSettings",
    "NameValuePair",
    "NetworkConfiguration",
    "NetworkSecurityGroupRule",
    "NfsMountConfiguration",
    "OSDisk",
    "OutputFile",
    "OutputFileBlobContainerDestination",
    "OutputFileDestination",
    "OutputFileUploadConfig",
    "PublicIpAddressConfiguration",
    "RecentBatchJob",
    "ResizeError",
    "ResourceFile",
    "RollingUpgradePolicy",
    "SecurityProfile",
    "ServiceArtifactReference",
    "UefiSettings",
    "UpgradePolicy",
    "UploadBatchServiceLogsContent",
    "UploadBatchServiceLogsResult",
    "UserAccount",
    "UserAssignedIdentity",
    "UserIdentity",
    "VMExtension",
    "VMExtensionInstanceView",
    "VirtualMachineConfiguration",
    "VirtualMachineInfo",
    "WindowsConfiguration",
    "WindowsUserConfiguration",
    "AccessScope",
    "AllocationState",
    "AutoUserScope",
    "BatchJobAction",
    "BatchJobPreparationTaskState",
    "BatchJobReleaseTaskState",
    "BatchJobScheduleState",
    "BatchJobState",
    "BatchNodeCommunicationMode",
    "BatchNodeDeallocationOption",
    "BatchNodeDisableSchedulingOption",
    "BatchNodeFillType",
    "BatchNodePlacementPolicyType",
    "BatchNodeRebootOption",
    "BatchNodeState",
    "BatchPoolIdentityType",
    "BatchPoolLifetimeOption",
    "BatchPoolState",
    "BatchStartTaskState",
    "BatchSubtaskState",
    "BatchTaskAddStatus",
    "BatchTaskExecutionResult",
    "BatchTaskState",
    "CachingType",
    "ContainerType",
    "ContainerWorkingDirectory",
    "DependencyAction",
    "DiffDiskPlacement",
    "DisableBatchJobOption",
    "DiskEncryptionTarget",
    "DynamicVNetAssignmentScope",
    "ElevationLevel",
    "ErrorCategory",
    "ImageVerificationType",
    "InboundEndpointProtocol",
    "IpAddressProvisioningType",
    "LoginMode",
    "NetworkSecurityGroupRuleAccess",
    "OSType",
    "OnAllBatchTasksComplete",
    "OnBatchTaskFailure",
    "OutputFileUploadCondition",
    "SchedulingState",
    "SecurityTypes",
    "StatusLevelTypes",
    "StorageAccountType",
    "UpgradeMode",
]
__all__.extend([p for p in _patch_all if p not in __all__])
_patch_sdk()



#The MIT License (MIT)
#Copyright (c) 2014 Microsoft Corporation

#Permission is hereby granted, free of charge, to any person obtaining a copy
#of this software and associated documentation files (the "Software"), to deal
#in the Software without restriction, including without limitation the rights
#to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#copies of the Software, and to permit persons to whom the Software is
#furnished to do so, subject to the following conditions:

#The above copyright notice and this permission notice shall be included in all
#copies or substantial portions of the Software.

#THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#SOFTWARE.

"""Internal class for global endpoint manager implementation in the Azure Cosmos database service.
"""

from six.moves.urllib.parse import urlparse
import threading
import azure.cosmos.constants as constants
import azure.cosmos.errors as errors
from azure.cosmos.location_cache import LocationCache

class _GlobalEndpointManager(object):
    """
    This internal class implements the logic for endpoint management for geo-replicated
    database accounts.
    """
    def __init__(self, client):
        self.Client = client
        self.EnableEndpointDiscovery = client.connection_policy.EnableEndpointDiscovery
        self.PreferredLocations = client.connection_policy.PreferredLocations
        self.DefaultEndpoint = client.url_connection
        self.refresh_time_interval_in_ms = self.get_refresh_time_interval_in_ms_stub()
        self.location_cache = LocationCache(self.PreferredLocations,
                                            self.DefaultEndpoint,
                                            self.EnableEndpointDiscovery,
                                            client.connection_policy.UseMultipleWriteLocations,
                                            self.refresh_time_interval_in_ms)
        self.refresh_needed = False
        self.refresh_lock = threading.RLock()
        self.last_refresh_time = 0

    def get_refresh_time_interval_in_ms_stub(self):
        return constants._Constants.DefaultUnavailableLocationExpirationTime

    def get_write_endpoint(self):
        return self.location_cache.get_write_endpoint()

    def get_read_endpoint(self):
        return self.location_cache.get_read_endpoint()

    def resolve_service_endpoint(self, request):
        return self.location_cache.resolve_service_endpoint(request)

    def mark_endpoint_unavailable_for_read(self, endpoint):
        self.location_cache.mark_endpoint_unavailable_for_read(endpoint)

    def mark_endpoint_unavailable_for_write(self, endpoint):
        self.location_cache.mark_endpoint_unavailable_for_write(endpoint)

    def get_ordered_write_endpoints(self):
        return self.location_cache.get_ordered_write_endpoints()

    def get_ordered_read_endpoints(self):
        return self.location_cache.get_ordered_read_endpoints()

    def can_use_multiple_write_locations(self, request):
        return self.location_cache.can_use_multiple_write_locations_for_request(request)

    def force_refresh(self, database_account):
        self.refresh_needed = True
        self.refresh_endpoint_list(database_account)

    def refresh_endpoint_list(self, database_account):
        with self.refresh_lock:
            # if refresh is not needed or refresh is already taking place, return
            if not self.refresh_needed:
                return
            try:
                self._refresh_endpoint_list_private(database_account)
            except Exception as e:
                raise e

    def _refresh_endpoint_list_private(self, database_account = None):
        if database_account :
            self.location_cache.perform_on_database_account_read(database_account)
            self.refresh_needed = False

        if self.location_cache.should_refresh_endpoints() and self.location_cache.current_time_millis() - self.last_refresh_time > self.refresh_time_interval_in_ms:
            if not database_account:
                database_account = self._GetDatabaseAccount()
                self.location_cache.perform_on_database_account_read(database_account)
                self.last_refresh_time = self.location_cache.current_time_millis()
                self.refresh_needed = False

    def _GetDatabaseAccount(self):
        """Gets the database account first by using the default endpoint, and if that doesn't returns
           use the endpoints for the preferred locations in the order they are specified to get 
           the database account.
        """
        try:
            database_account = self._GetDatabaseAccountStub(self.DefaultEndpoint)
            return database_account
        # If for any reason(non-globaldb related), we are not able to get the database account from the above call to GetDatabaseAccount,
        # we would try to get this information from any of the preferred locations that the user might have specified(by creating a locational endpoint)
        # and keeping eating the exception until we get the database account and return None at the end, if we are not able to get that info from any endpoints
        except errors.HTTPFailure:
            for location_name in self.PreferredLocations:
                locational_endpoint = _GlobalEndpointManager.GetLocationalEndpoint(self.DefaultEndpoint, location_name)
                try:
                    database_account = self._GetDatabaseAccountStub(locational_endpoint)
                    return database_account
                except errors.HTTPFailure:
                    pass

            return None

    def _GetDatabaseAccountStub(self, endpoint):
        """Stub for getting database account from the client
           which can be used for mocking purposes as well.
        """
        return self.Client.GetDatabaseAccount(endpoint)

    @staticmethod
    def GetLocationalEndpoint(default_endpoint, location_name):
        # For default_endpoint like 'https://contoso.documents.azure.com:443/' parse it to generate URL format
        # This default_endpoint should be global endpoint(and cannot be a locational endpoint) and we agreed to document that
        endpoint_url = urlparse(default_endpoint)

        # hostname attribute in endpoint_url will return 'contoso.documents.azure.com'
        if endpoint_url.hostname is not None:
            hostname_parts = str(endpoint_url.hostname).lower().split('.')
            if hostname_parts is not None:
                # global_database_account_name will return 'contoso'
                global_database_account_name = hostname_parts[0]
            
                # Prepare the locational_database_account_name as contoso-EastUS for location_name 'East US'
                locational_database_account_name = global_database_account_name + '-' + location_name.replace(' ', '')
            
                # Replace 'contoso' with 'contoso-EastUS' and return locational_endpoint as https://contoso-EastUS.documents.azure.com:443/
                locational_endpoint = default_endpoint.lower().replace(global_database_account_name, locational_database_account_name, 1)  
                return locational_endpoint
        
        return None



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
from typing import TYPE_CHECKING
import warnings

from azure.core.exceptions import ClientAuthenticationError, HttpResponseError, ResourceExistsError, ResourceNotFoundError, map_error
from azure.core.paging import ItemPaged
from azure.core.pipeline import PipelineResponse
from azure.core.pipeline.transport import HttpRequest, HttpResponse
from azure.mgmt.core.exceptions import ARMErrorFormat

from .. import models as _models

if TYPE_CHECKING:
    # pylint: disable=unused-import,ungrouped-imports
    from typing import Any, Callable, Dict, Generic, Iterable, Optional, TypeVar

    T = TypeVar('T')
    ClsType = Optional[Callable[[PipelineResponse[HttpRequest, HttpResponse], T, Dict[str, Any]], Any]]

class RecommendationsOperations(object):
    """RecommendationsOperations operations.

    You should not instantiate this class directly. Instead, you should create a Client instance that
    instantiates it for you and attaches it as an attribute.

    :ivar models: Alias to model classes used in this operation group.
    :type models: ~azure.mgmt.advisor.models
    :param client: Client for service requests.
    :param config: Configuration of service client.
    :param serializer: An object model serializer.
    :param deserializer: An object model deserializer.
    """

    models = _models

    def __init__(self, client, config, serializer, deserializer):
        self._client = client
        self._serialize = serializer
        self._deserialize = deserializer
        self._config = config

    def generate(
        self,
        **kwargs  # type: Any
    ):
        # type: (...) -> None
        """Initiates the recommendation generation or computation process for a subscription. This
        operation is asynchronous. The generated recommendations are stored in a cache in the Advisor
        service.

        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: None, or the result of cls(response)
        :rtype: None
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType[None]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))
        api_version = "2020-01-01"

        # Construct URL
        url = self.generate.metadata['url']  # type: ignore
        path_format_arguments = {
            'subscriptionId': self._serialize.url("self._config.subscription_id", self._config.subscription_id, 'str'),
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}  # type: Dict[str, Any]
        query_parameters['api-version'] = self._serialize.query("api_version", api_version, 'str')

        # Construct headers
        header_parameters = {}  # type: Dict[str, Any]

        request = self._client.post(url, query_parameters, header_parameters)
        pipeline_response = self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [202]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        response_headers = {}
        response_headers['Location']=self._deserialize('str', response.headers.get('Location'))
        response_headers['Retry-After']=self._deserialize('str', response.headers.get('Retry-After'))

        if cls:
            return cls(pipeline_response, None, response_headers)

    generate.metadata = {'url': '/subscriptions/{subscriptionId}/providers/Microsoft.Advisor/generateRecommendations'}  # type: ignore

    def get_generate_status(
        self,
        operation_id,  # type: str
        **kwargs  # type: Any
    ):
        # type: (...) -> None
        """Retrieves the status of the recommendation computation or generation process. Invoke this API
        after calling the generation recommendation. The URI of this API is returned in the Location
        field of the response header.

        :param operation_id: The operation ID, which can be found from the Location field in the
         generate recommendation response header.
        :type operation_id: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: None, or the result of cls(response)
        :rtype: None
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType[None]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))
        api_version = "2020-01-01"

        # Construct URL
        url = self.get_generate_status.metadata['url']  # type: ignore
        path_format_arguments = {
            'subscriptionId': self._serialize.url("self._config.subscription_id", self._config.subscription_id, 'str'),
            'operationId': self._serialize.url("operation_id", operation_id, 'str'),
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}  # type: Dict[str, Any]
        query_parameters['api-version'] = self._serialize.query("api_version", api_version, 'str')

        # Construct headers
        header_parameters = {}  # type: Dict[str, Any]

        request = self._client.get(url, query_parameters, header_parameters)
        pipeline_response = self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [202, 204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        if cls:
            return cls(pipeline_response, None, {})

    get_generate_status.metadata = {'url': '/subscriptions/{subscriptionId}/providers/Microsoft.Advisor/generateRecommendations/{operationId}'}  # type: ignore

    def list(
        self,
        filter=None,  # type: Optional[str]
        top=None,  # type: Optional[int]
        skip_token=None,  # type: Optional[str]
        **kwargs  # type: Any
    ):
        # type: (...) -> Iterable["_models.ResourceRecommendationBaseListResult"]
        """Obtains cached recommendations for a subscription. The recommendations are generated or
        computed by invoking generateRecommendations.

        :param filter: The filter to apply to the recommendations.:code:`<br>`Filter can be applied to
         properties ['ResourceId', 'ResourceGroup', 'RecommendationTypeGuid', '\ `Category
         <#category>`_\ '] with operators ['eq', 'and', 'or'].:code:`<br>`Example::code:`<br>`-
         $filter=Category eq 'Cost' and ResourceGroup eq 'MyResourceGroup'.
        :type filter: str
        :param top: The number of recommendations per page if a paged version of this API is being
         used.
        :type top: int
        :param skip_token: The page-continuation token to use with a paged version of this API.
        :type skip_token: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: An iterator like instance of either ResourceRecommendationBaseListResult or the result of cls(response)
        :rtype: ~azure.core.paging.ItemPaged[~azure.mgmt.advisor.models.ResourceRecommendationBaseListResult]
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.ResourceRecommendationBaseListResult"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))
        api_version = "2020-01-01"
        accept = "application/json"

        def prepare_request(next_link=None):
            # Construct headers
            header_parameters = {}  # type: Dict[str, Any]
            header_parameters['Accept'] = self._serialize.header("accept", accept, 'str')

            if not next_link:
                # Construct URL
                url = self.list.metadata['url']  # type: ignore
                path_format_arguments = {
                    'subscriptionId': self._serialize.url("self._config.subscription_id", self._config.subscription_id, 'str'),
                }
                url = self._client.format_url(url, **path_format_arguments)
                # Construct parameters
                query_parameters = {}  # type: Dict[str, Any]
                query_parameters['api-version'] = self._serialize.query("api_version", api_version, 'str')
                if filter is not None:
                    query_parameters['$filter'] = self._serialize.query("filter", filter, 'str')
                if top is not None:
                    query_parameters['$top'] = self._serialize.query("top", top, 'int')
                if skip_token is not None:
                    query_parameters['$skipToken'] = self._serialize.query("skip_token", skip_token, 'str')

                request = self._client.get(url, query_parameters, header_parameters)
            else:
                url = next_link
                query_parameters = {}  # type: Dict[str, Any]
                request = self._client.get(url, query_parameters, header_parameters)
            return request

        def extract_data(pipeline_response):
            deserialized = self._deserialize('ResourceRecommendationBaseListResult', pipeline_response)
            list_of_elem = deserialized.value
            if cls:
                list_of_elem = cls(list_of_elem)
            return deserialized.next_link or None, iter(list_of_elem)

        def get_next(next_link=None):
            request = prepare_request(next_link)

            pipeline_response = self._client._pipeline.run(request, stream=False, **kwargs)
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                raise HttpResponseError(response=response, error_format=ARMErrorFormat)

            return pipeline_response

        return ItemPaged(
            get_next, extract_data
        )
    list.metadata = {'url': '/subscriptions/{subscriptionId}/providers/Microsoft.Advisor/recommendations'}  # type: ignore

    def get(
        self,
        resource_uri,  # type: str
        recommendation_id,  # type: str
        **kwargs  # type: Any
    ):
        # type: (...) -> "_models.ResourceRecommendationBase"
        """Obtains details of a cached recommendation.

        :param resource_uri: The fully qualified Azure Resource Manager identifier of the resource to
         which the recommendation applies.
        :type resource_uri: str
        :param recommendation_id: The recommendation ID.
        :type recommendation_id: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: ResourceRecommendationBase, or the result of cls(response)
        :rtype: ~azure.mgmt.advisor.models.ResourceRecommendationBase
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.ResourceRecommendationBase"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))
        api_version = "2020-01-01"
        accept = "application/json"

        # Construct URL
        url = self.get.metadata['url']  # type: ignore
        path_format_arguments = {
            'resourceUri': self._serialize.url("resource_uri", resource_uri, 'str'),
            'recommendationId': self._serialize.url("recommendation_id", recommendation_id, 'str'),
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}  # type: Dict[str, Any]
        query_parameters['api-version'] = self._serialize.query("api_version", api_version, 'str')

        # Construct headers
        header_parameters = {}  # type: Dict[str, Any]
        header_parameters['Accept'] = self._serialize.header("accept", accept, 'str')

        request = self._client.get(url, query_parameters, header_parameters)
        pipeline_response = self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        deserialized = self._deserialize('ResourceRecommendationBase', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized
    get.metadata = {'url': '/{resourceUri}/providers/Microsoft.Advisor/recommendations/{recommendationId}'}  # type: ignore



# -*- coding: utf-8 -*-
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------
from enum import Enum

class ExpiryOptionType(Enum):
    never_expire = "NeverExpire"
    relative_to_now = "RelativeToNow"
    relative_to_creation_date = "RelativeToCreationDate"
    absolute = "Absolute"


# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


class Create(AAZCommand):
    """The operation to create the run command.

    :example: Create a run command.
        az vm run-command create --resource-group "myResourceGroup" --location "West US" --async-execution false --parameters arg1=value1 arg2=value2 --run-as-password "<runAsPassword>" --run-as-user "user1" --script "Write-Host Hello World!" --timeout-in-seconds 3600 --run-command-name "myRunCommand" --vm-name "myVM"

    :example: Create a run command with uploading script output stream to Azure storage blob (SAS URI).
        az vm run-command create --resource-group "myResourceGroup" --location "West US" --script "Write-Host Hello World!" --run-command-name "myRunCommand" --vm-name "myVM" --output-blob-uri "https://my-blob-uri"
    """

    _aaz_info = {
        "version": "2024-11-01",
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.compute/virtualmachines/{}/runcommands/{}", "2024-11-01"],
        ]
    }

    AZ_SUPPORT_NO_WAIT = True

    def _handler(self, command_args):
        super()._handler(command_args)
        return self.build_lro_poller(self._execute_operations, self._output)

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.resource_group = AAZResourceGroupNameArg(
            required=True,
        )
        _args_schema.run_command_name = AAZStrArg(
            options=["-n", "--name", "--run-command-name"],
            help="The name of the VirtualMachineRunCommand",
            required=True,
        )
        _args_schema.vm_name = AAZStrArg(
            options=["--vm-name"],
            help="The name of the VirtualMachine",
            required=True,
        )
        _args_schema.location = AAZResourceLocationArg(
            help="Location. Values from: `az account list-locations`. You can configure the default location using `az configure --defaults location=<location>`.",
            required=True,
            fmt=AAZResourceLocationArgFormat(
                resource_group_arg="resource_group",
            ),
        )
        _args_schema.async_execution = AAZBoolArg(
            options=["--async-execution"],
            help="Optional. If set to true, provisioning will complete as soon as the script starts and will not wait for script to complete.",
        )
        _args_schema.error_blob_uri = AAZStrArg(
            options=["--error-blob-uri"],
            help="Specify the Azure storage blob where script error stream will be uploaded.",
        )
        _args_schema.output_blob_uri = AAZStrArg(
            options=["--output-blob-uri"],
            help="Specify the Azure storage blob (SAS URI) where script output stream will be uploaded.",
        )
        _args_schema.parameters = AAZListArg(
            options=["--parameters"],
            help="The parameters used by the script.",
        )
        _args_schema.protected_parameters = AAZListArg(
            options=["--protected-parameters"],
            help="The parameters used by the script.",
        )
        _args_schema.run_as_password = AAZStrArg(
            options=["--run-as-password"],
            help="Password if needed for using run-as-user parameter. It will be encrypted and not logged",
        )
        _args_schema.run_as_user = AAZStrArg(
            options=["--run-as-user"],
            help="By default script process runs under system/root user. Specify custom user to host the process.",
        )
        _args_schema.command_id = AAZStrArg(
            options=["--command-id"],
            help="Specifies a commandId of predefined built-in script.",
        )
        _args_schema.script = AAZStrArg(
            options=["--script"],
            help="Specifies the script content to be executed on the VM.",
        )
        _args_schema.script_uri = AAZStrArg(
            options=["--script-uri"],
            help="Specifies the script download location. It can be either SAS URI of an Azure storage blob with read access or public URI.",
        )
        _args_schema.timeout_in_seconds = AAZIntArg(
            options=["--timeout-in-seconds"],
            help="The timeout in seconds to execute the run command.",
        )
        _args_schema.tags = AAZDictArg(
            options=["--tags"],
            help="Space-separated tags: key[=value] [key[=value] ...].",
        )

        parameters = cls._args_schema.parameters
        parameters.Element = AAZObjectArg()
        cls._build_args_run_command_input_parameter_create(parameters.Element)

        protected_parameters = cls._args_schema.protected_parameters
        protected_parameters.Element = AAZObjectArg()
        cls._build_args_run_command_input_parameter_create(protected_parameters.Element)

        tags = cls._args_schema.tags
        tags.Element = AAZStrArg()

        # define Arg Group "Properties"

        # define Arg Group "Source"
        return cls._args_schema

    _args_run_command_input_parameter_create = None

    @classmethod
    def _build_args_run_command_input_parameter_create(cls, _schema):
        if cls._args_run_command_input_parameter_create is not None:
            _schema.name = cls._args_run_command_input_parameter_create.name
            _schema.value = cls._args_run_command_input_parameter_create.value
            return

        cls._args_run_command_input_parameter_create = AAZObjectArg()

        run_command_input_parameter_create = cls._args_run_command_input_parameter_create
        run_command_input_parameter_create.name = AAZStrArg(
            options=["name"],
            help="The run command parameter name.",
            required=True,
        )
        run_command_input_parameter_create.value = AAZStrArg(
            options=["value"],
            help="The run command parameter value.",
            required=True,
        )

        _schema.name = cls._args_run_command_input_parameter_create.name
        _schema.value = cls._args_run_command_input_parameter_create.value

    _args_run_command_managed_identity_create = None

    @classmethod
    def _build_args_run_command_managed_identity_create(cls, _schema):
        if cls._args_run_command_managed_identity_create is not None:
            _schema.client_id = cls._args_run_command_managed_identity_create.client_id
            _schema.object_id = cls._args_run_command_managed_identity_create.object_id
            return

        cls._args_run_command_managed_identity_create = AAZObjectArg()

        run_command_managed_identity_create = cls._args_run_command_managed_identity_create
        run_command_managed_identity_create.client_id = AAZStrArg(
            options=["client-id"],
            help="Client Id (GUID value) of the user-assigned managed identity. ObjectId should not be used if this is provided.",
        )
        run_command_managed_identity_create.object_id = AAZStrArg(
            options=["object-id"],
            help="Object Id (GUID value) of the user-assigned managed identity. ClientId should not be used if this is provided.",
        )

        _schema.client_id = cls._args_run_command_managed_identity_create.client_id
        _schema.object_id = cls._args_run_command_managed_identity_create.object_id

    def _execute_operations(self):
        self.pre_operations()
        yield self.VirtualMachineRunCommandsCreateOrUpdate(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result

    class VirtualMachineRunCommandsCreateOrUpdate(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [202]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    self.on_200_201,
                    self.on_error,
                    lro_options={"final-state-via": "location"},
                    path_format_arguments=self.url_parameters,
                )
            if session.http_response.status_code in [200, 201]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    self.on_200_201,
                    self.on_error,
                    lro_options={"final-state-via": "location"},
                    path_format_arguments=self.url_parameters,
                )

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Compute/virtualMachines/{vmName}/runCommands/{runCommandName}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "PUT"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "runCommandName", self.ctx.args.run_command_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
                **self.serialize_url_param(
                    "vmName", self.ctx.args.vm_name,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2024-11-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Content-Type", "application/json",
                ),
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        @property
        def content(self):
            _content_value, _builder = self.new_content_builder(
                self.ctx.args,
                typ=AAZObjectType,
                typ_kwargs={"flags": {"required": True, "client_flatten": True}}
            )
            _builder.set_prop("location", AAZStrType, ".location", typ_kwargs={"flags": {"required": True}})
            _builder.set_prop("properties", AAZObjectType, typ_kwargs={"flags": {"client_flatten": True}})
            _builder.set_prop("tags", AAZDictType, ".tags")

            properties = _builder.get(".properties")
            if properties is not None:
                properties.set_prop("asyncExecution", AAZBoolType, ".async_execution")
                properties.set_prop("errorBlobUri", AAZStrType, ".error_blob_uri")
                properties.set_prop("outputBlobUri", AAZStrType, ".output_blob_uri")
                properties.set_prop("parameters", AAZListType, ".parameters")
                properties.set_prop("protectedParameters", AAZListType, ".protected_parameters")
                properties.set_prop("runAsPassword", AAZStrType, ".run_as_password")
                properties.set_prop("runAsUser", AAZStrType, ".run_as_user")
                properties.set_prop("source", AAZObjectType)
                properties.set_prop("timeoutInSeconds", AAZIntType, ".timeout_in_seconds")

            parameters = _builder.get(".properties.parameters")
            if parameters is not None:
                _CreateHelper._build_schema_run_command_input_parameter_create(parameters.set_elements(AAZObjectType, "."))

            protected_parameters = _builder.get(".properties.protectedParameters")
            if protected_parameters is not None:
                _CreateHelper._build_schema_run_command_input_parameter_create(protected_parameters.set_elements(AAZObjectType, "."))

            source = _builder.get(".properties.source")
            if source is not None:
                source.set_prop("commandId", AAZStrType, ".command_id")
                source.set_prop("script", AAZStrType, ".script")
                source.set_prop("scriptUri", AAZStrType, ".script_uri")

            tags = _builder.get(".tags")
            if tags is not None:
                tags.set_elements(AAZStrType, ".")

            return self.serialize_content(_content_value)

        def on_200_201(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200_201
            )

        _schema_on_200_201 = None

        @classmethod
        def _build_schema_on_200_201(cls):
            if cls._schema_on_200_201 is not None:
                return cls._schema_on_200_201

            cls._schema_on_200_201 = AAZObjectType()

            _schema_on_200_201 = cls._schema_on_200_201
            _schema_on_200_201.id = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200_201.location = AAZStrType(
                flags={"required": True},
            )
            _schema_on_200_201.name = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200_201.properties = AAZObjectType(
                flags={"client_flatten": True},
            )
            _schema_on_200_201.system_data = AAZObjectType(
                serialized_name="systemData",
                flags={"read_only": True},
            )
            _schema_on_200_201.tags = AAZDictType()
            _schema_on_200_201.type = AAZStrType(
                flags={"read_only": True},
            )

            properties = cls._schema_on_200_201.properties
            properties.async_execution = AAZBoolType(
                serialized_name="asyncExecution",
            )
            properties.error_blob_managed_identity = AAZObjectType(
                serialized_name="errorBlobManagedIdentity",
            )
            _CreateHelper._build_schema_run_command_managed_identity_read(properties.error_blob_managed_identity)
            properties.error_blob_uri = AAZStrType(
                serialized_name="errorBlobUri",
            )
            properties.instance_view = AAZObjectType(
                serialized_name="instanceView",
                flags={"read_only": True},
            )
            properties.output_blob_managed_identity = AAZObjectType(
                serialized_name="outputBlobManagedIdentity",
            )
            _CreateHelper._build_schema_run_command_managed_identity_read(properties.output_blob_managed_identity)
            properties.output_blob_uri = AAZStrType(
                serialized_name="outputBlobUri",
            )
            properties.parameters = AAZListType()
            properties.protected_parameters = AAZListType(
                serialized_name="protectedParameters",
            )
            properties.provisioning_state = AAZStrType(
                serialized_name="provisioningState",
                flags={"read_only": True},
            )
            properties.run_as_password = AAZStrType(
                serialized_name="runAsPassword",
            )
            properties.run_as_user = AAZStrType(
                serialized_name="runAsUser",
            )
            properties.source = AAZObjectType()
            properties.timeout_in_seconds = AAZIntType(
                serialized_name="timeoutInSeconds",
            )
            properties.treat_failure_as_deployment_failure = AAZBoolType(
                serialized_name="treatFailureAsDeploymentFailure",
            )

            instance_view = cls._schema_on_200_201.properties.instance_view
            instance_view.end_time = AAZStrType(
                serialized_name="endTime",
            )
            instance_view.error = AAZStrType()
            instance_view.execution_message = AAZStrType(
                serialized_name="executionMessage",
            )
            instance_view.execution_state = AAZStrType(
                serialized_name="executionState",
            )
            instance_view.exit_code = AAZIntType(
                serialized_name="exitCode",
            )
            instance_view.output = AAZStrType()
            instance_view.start_time = AAZStrType(
                serialized_name="startTime",
            )
            instance_view.statuses = AAZListType()

            statuses = cls._schema_on_200_201.properties.instance_view.statuses
            statuses.Element = AAZObjectType()

            _element = cls._schema_on_200_201.properties.instance_view.statuses.Element
            _element.code = AAZStrType()
            _element.display_status = AAZStrType(
                serialized_name="displayStatus",
            )
            _element.level = AAZStrType()
            _element.message = AAZStrType()
            _element.time = AAZStrType()

            parameters = cls._schema_on_200_201.properties.parameters
            parameters.Element = AAZObjectType()
            _CreateHelper._build_schema_run_command_input_parameter_read(parameters.Element)

            protected_parameters = cls._schema_on_200_201.properties.protected_parameters
            protected_parameters.Element = AAZObjectType()
            _CreateHelper._build_schema_run_command_input_parameter_read(protected_parameters.Element)

            source = cls._schema_on_200_201.properties.source
            source.command_id = AAZStrType(
                serialized_name="commandId",
            )
            source.script = AAZStrType()
            source.script_uri = AAZStrType(
                serialized_name="scriptUri",
            )
            source.script_uri_managed_identity = AAZObjectType(
                serialized_name="scriptUriManagedIdentity",
            )
            _CreateHelper._build_schema_run_command_managed_identity_read(source.script_uri_managed_identity)

            system_data = cls._schema_on_200_201.system_data
            system_data.created_at = AAZStrType(
                serialized_name="createdAt",
            )
            system_data.created_by = AAZStrType(
                serialized_name="createdBy",
            )
            system_data.created_by_type = AAZStrType(
                serialized_name="createdByType",
            )
            system_data.last_modified_at = AAZStrType(
                serialized_name="lastModifiedAt",
            )
            system_data.last_modified_by = AAZStrType(
                serialized_name="lastModifiedBy",
            )
            system_data.last_modified_by_type = AAZStrType(
                serialized_name="lastModifiedByType",
            )

            tags = cls._schema_on_200_201.tags
            tags.Element = AAZStrType()

            return cls._schema_on_200_201


class _CreateHelper:
    """Helper class for Create"""

    @classmethod
    def _build_schema_run_command_input_parameter_create(cls, _builder):
        if _builder is None:
            return
        _builder.set_prop("name", AAZStrType, ".name", typ_kwargs={"flags": {"required": True}})
        _builder.set_prop("value", AAZStrType, ".value", typ_kwargs={"flags": {"required": True}})

    @classmethod
    def _build_schema_run_command_managed_identity_create(cls, _builder):
        if _builder is None:
            return
        _builder.set_prop("clientId", AAZStrType, ".client_id")
        _builder.set_prop("objectId", AAZStrType, ".object_id")

    _schema_run_command_input_parameter_read = None

    @classmethod
    def _build_schema_run_command_input_parameter_read(cls, _schema):
        if cls._schema_run_command_input_parameter_read is not None:
            _schema.name = cls._schema_run_command_input_parameter_read.name
            _schema.value = cls._schema_run_command_input_parameter_read.value
            return

        cls._schema_run_command_input_parameter_read = _schema_run_command_input_parameter_read = AAZObjectType()

        run_command_input_parameter_read = _schema_run_command_input_parameter_read
        run_command_input_parameter_read.name = AAZStrType(
            flags={"required": True},
        )
        run_command_input_parameter_read.value = AAZStrType(
            flags={"required": True},
        )

        _schema.name = cls._schema_run_command_input_parameter_read.name
        _schema.value = cls._schema_run_command_input_parameter_read.value

    _schema_run_command_managed_identity_read = None

    @classmethod
    def _build_schema_run_command_managed_identity_read(cls, _schema):
        if cls._schema_run_command_managed_identity_read is not None:
            _schema.client_id = cls._schema_run_command_managed_identity_read.client_id
            _schema.object_id = cls._schema_run_command_managed_identity_read.object_id
            return

        cls._schema_run_command_managed_identity_read = _schema_run_command_managed_identity_read = AAZObjectType()

        run_command_managed_identity_read = _schema_run_command_managed_identity_read
        run_command_managed_identity_read.client_id = AAZStrType(
            serialized_name="clientId",
        )
        run_command_managed_identity_read.object_id = AAZStrType(
            serialized_name="objectId",
        )

        _schema.client_id = cls._schema_run_command_managed_identity_read.client_id
        _schema.object_id = cls._schema_run_command_managed_identity_read.object_id


__all__ = ["Create"]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import time
import os
import urllib
import urllib3
import certifi
from datetime import datetime

from knack.log import get_logger

from azure.cli.core.azclierror import (RequiredArgumentMissingError, ValidationError, ResourceNotFoundError,
                                       CLIInternalError)
from azure.cli.core.commands.parameters import get_subscription_locations
from azure.cli.core.util import should_disable_connection_verify, send_raw_request
from azure.cli.core.commands.client_factory import get_subscription_id

from azure.mgmt.core.tools import parse_resource_id, is_valid_resource_id, resource_id

from ._client_factory import web_client_factory, providers_client_factory
from ._constants import LOGICAPP_KIND, FUNCTIONAPP_KIND, LINUXAPP_KIND

logger = get_logger(__name__)

REQUESTS_CA_BUNDLE = "REQUESTS_CA_BUNDLE"


# get a RID when the name may be an RID
def get_resource_id(cmd, name, resource_group, namespace, type, **rid_kwargs):  # pylint: disable=redefined-builtin
    if is_valid_resource_id(name):
        return name
    return resource_id(subscription=get_subscription_id(cmd.cli_ctx),
                       resource_group=resource_group,
                       name=name,
                       namespace=namespace,
                       type=type,
                       **rid_kwargs)


# get a resource's name and group when the name may be an RID
def get_resource_name_and_group(cmd, name, resource_group, namespace, type, **rid_kwargs):  # pylint: disable=redefined-builtin
    rid = get_resource_id(cmd, name, resource_group, namespace, type, **rid_kwargs)
    rid_parsed = parse_resource_id(rid)
    return rid_parsed.get("name"), rid_parsed.get("resource_group"), rid


def str2bool(v):
    if v == 'true':
        retval = True
    elif v == 'false':
        retval = False
    else:
        retval = None
    return retval


def _normalize_sku(sku):
    sku = sku.upper()
    if sku == 'FREE':
        return 'F1'
    if sku == 'SHARED':
        return 'D1'
    return sku


def get_sku_tier(name):  # pylint: disable=too-many-return-statements
    name = name.upper()
    if name in ['F1', 'FREE']:
        return 'FREE'
    if name in ['D1', "SHARED"]:
        return 'SHARED'
    if name in ['B1', 'B2', 'B3', 'BASIC']:
        return 'BASIC'
    if name in ['S1', 'S2', 'S3']:
        return 'STANDARD'
    if name in ['P1', 'P2', 'P3']:
        return 'PREMIUM'
    if name in ['P1V2', 'P2V2', 'P3V2']:
        return 'PREMIUMV2'
    if name in ['P0V3']:
        return 'PREMIUM0V3'
    if name in ['P1V3', 'P2V3', 'P3V3']:
        return 'PREMIUMV3'
    if name in ['P1MV3', 'P2MV3', 'P3MV3', 'P4MV3', 'P5MV3']:
        return 'PREMIUMMV3'
    if name in ['P0V4', 'P1V4', 'P2V4', 'P3V4']:
        return 'PREMIUMV4'
    if name in ['P1MV4', 'P2MV4', 'P3MV4', 'P4MV4', 'P5MV4']:
        return 'PREMIUMMV4'
    if name in ['PC2', 'PC3', 'PC4']:
        return 'PremiumContainer'
    if name in ['EP1', 'EP2', 'EP3']:
        return 'ElasticPremium'
    if name in ['I1V2', 'I2V2', 'I3V2', 'I4V2', 'I5V2', 'I6V2']:
        return 'IsolatedV2'
    if name in ['I1MV2', 'I2MV2', 'I3MV2', 'I4MV2', 'I5MV2']:
        return 'IsolatedMV2'
    if name in ['WS1', 'WS2', 'WS3']:
        return 'WorkflowStandard'
    raise ValidationError("Invalid sku(pricing tier), please refer to command help for valid values")


# Deprecated; Do not use
# Keeping this for now so that we don't break extensions that use it
def get_sku_name(tier):
    return get_sku_tier(name=tier)


def is_sku_tier_enabled_for_managed_instance(sku_tier):
    sku_tier = sku_tier.upper()
    enabled_skus = ['PREMIUMV4', 'PREMIUMMV4']
    return sku_tier in enabled_skus


# resource is client.web_apps for webapps, client.app_service_plans for ASPs, etc.
def get_resource_if_exists(resource, **kwargs):
    from azure.core.exceptions import ResourceNotFoundError as R

    try:
        return resource.get(**kwargs)
    except (R, ValueError):
        return None


def normalize_sku_for_staticapp(sku):
    if sku.lower() == 'free':
        return 'Free'
    if sku.lower() == 'standard':
        return 'Standard'
    if sku.lower() == 'dedicated':
        return 'Dedicated'
    raise ValidationError("Invalid sku(pricing tier), please refer to command help for valid values")


def retryable_method(retries=3, interval_sec=5, excpt_type=Exception):
    def decorate(func):
        def call(*args, **kwargs):
            current_retry = retries
            while True:
                try:
                    return func(*args, **kwargs)
                except excpt_type:  # pylint: disable=broad-except
                    current_retry -= 1
                    if current_retry <= 0:
                        raise
                time.sleep(interval_sec)
        return call
    return decorate


def register_app_provider(cmd):
    from azure.mgmt.resource.resources.models import ProviderRegistrationRequest, ProviderConsentDefinition

    namespace = "Microsoft.App"
    providers_client = providers_client_factory(cmd.cli_ctx)
    is_registered = False
    try:
        registration_state = providers_client.get(namespace).registration_state
        is_registered = registration_state and registration_state.lower() == 'registered'
    except Exception:  # pylint: disable=broad-except
        logger.warning("An error occurred while trying to get the registration state of the '%s' provider.", namespace)

    if not is_registered:
        try:
            logger.warning("Registering the '%s' provider.", namespace)
            properties = ProviderRegistrationRequest(
                third_party_provider_consent=ProviderConsentDefinition(consent_to_authorization=True))
            providers_client.register(namespace, properties=properties)
            timeout_secs = 120
            start_time = datetime.now()
            while not is_registered:
                time.sleep(5)
                registration_state = providers_client.get(namespace).registration_state
                is_registered = registration_state and registration_state.lower() == 'registered'
                if not is_registered and (datetime.now() - start_time).seconds >= timeout_secs:
                    raise CLIInternalError("Timed out waiting for the '%s' provider to register." % namespace)
            logger.warning("Successfully registered the '%s' provider.", namespace)
        except Exception as ex:  # pylint: disable=broad-except
            logger.warning("An error occurred while trying to register the '%s' provider: %s", namespace, str(ex))


def raise_missing_token_suggestion():
    pat_documentation = "https://help.github.com/en/articles/creating-a-personal-access-token-for-the-command-line"
    raise RequiredArgumentMissingError("GitHub access token is required to authenticate to your repositories. "
                                       "If you need to create a Github Personal Access Token, "
                                       "please run with the '--login-with-github' flag or follow "
                                       "the steps found at the following link:\n{0}".format(pat_documentation))


def raise_missing_ado_token_suggestion():
    pat_documentation = ("https://learn.microsoft.com/en-us/azure/devops/organizations/accounts/use-personal-access-"
                         "tokens-to-authenticate?view=azure-devops&tabs=Windows#create-a-pat")
    raise RequiredArgumentMissingError("If this repo is an Azure Dev Ops repo, please provide a Personal Access Token."
                                       "Please run with the '--login-with-ado' flag or follow "
                                       "the steps found at the following link:\n{0}".format(pat_documentation))


def _get_location_from_resource_group(cli_ctx, resource_group_name):
    from azure.cli.core.commands.client_factory import get_mgmt_service_client
    from azure.cli.core.profiles import ResourceType
    client = get_mgmt_service_client(cli_ctx, ResourceType.MGMT_RESOURCE_RESOURCES)
    group = client.resource_groups.get(resource_group_name)
    return group.location


def is_centauri_functionapp(cmd, resource_group, name):
    client = web_client_factory(cmd.cli_ctx)
    functionapp = client.web_apps.get(resource_group, name)
    return functionapp.managed_environment_id is not None


def is_flex_functionapp(cli_ctx, resource_group, name):
    app = get_raw_functionapp(cli_ctx, resource_group, name)
    if app["properties"]["serverFarmId"] is None:
        return False
    sku = app["properties"]["sku"]
    return sku and sku.lower() == 'flexconsumption'


def _list_app(cli_ctx, resource_group_name=None):
    client = web_client_factory(cli_ctx)
    if resource_group_name:
        result = list(client.web_apps.list_by_resource_group(resource_group_name))
    else:
        result = list(client.web_apps.list())
    for webapp in result:
        _rename_server_farm_props(webapp)
    return result


def _rename_server_farm_props(webapp):
    # Should be renamed in SDK in a future release
    setattr(webapp, 'app_service_plan_id', webapp.server_farm_id)
    del webapp.server_farm_id
    return webapp


def get_raw_functionapp(cli_ctx, resource_group_name, name):
    site_url_base = '/subscriptions/{}/resourceGroups/{}/providers/Microsoft.Web/sites/{}?api-version={}'
    subscription_id = get_subscription_id(cli_ctx)
    site_url = site_url_base.format(subscription_id, resource_group_name, name, '2023-12-01')
    request_url = cli_ctx.cloud.endpoints.resource_manager + site_url
    response = send_raw_request(cli_ctx, "GET", request_url)
    return response.json()


def _get_location_from_webapp(client, resource_group_name, webapp):
    webapp = client.web_apps.get(resource_group_name, webapp)
    if not webapp:
        raise ResourceNotFoundError("'{}' app doesn't exist".format(webapp))
    return webapp.location


def _normalize_flex_location(location):
    return location.lower().replace(" ", "")


# can't just normalize locations with location.lower().replace(" ", "") because of UAE/UK regions
def _normalize_location(cmd, location):
    location = location.lower()
    locations = get_subscription_locations(cmd.cli_ctx)
    for loc in locations:
        if loc.display_name.lower() == location or loc.name.lower() == location:
            return loc.name
    return location


def _remove_list_duplicates(webapp):
    outbound_ips = webapp.possible_outbound_ip_addresses.split(',')
    outbound_ips_list = list(dict.fromkeys(outbound_ips))
    outbound_ips_list.sort()
    outbound_ips = ','.join(outbound_ips_list)
    del webapp.possible_outbound_ip_addresses
    setattr(webapp, 'possible_outbound_ip_addresses', outbound_ips)


def get_pool_manager(url):
    proxies = urllib.request.getproxies()
    bypass_proxy = urllib.request.proxy_bypass(urllib.parse.urlparse(url).hostname)

    if 'https' in proxies and not bypass_proxy:
        proxy = urllib.parse.urlparse(proxies['https'])

        if proxy.username and proxy.password:
            proxy_headers = urllib3.util.make_headers(proxy_basic_auth='{0}:{1}'.format(proxy.username, proxy.password))
            logger.debug('Setting proxy-authorization header for basic auth')
        else:
            proxy_headers = None

        logger.info('Using proxy for app service tunnel connection')
        http = urllib3.ProxyManager(proxy.geturl(), proxy_headers=proxy_headers)
    else:
        http = urllib3.PoolManager()

    if should_disable_connection_verify():
        http.connection_pool_kw['cert_reqs'] = 'CERT_NONE'
    else:
        http.connection_pool_kw['cert_reqs'] = 'CERT_REQUIRED'
        if REQUESTS_CA_BUNDLE in os.environ:
            ca_bundle_file = os.environ[REQUESTS_CA_BUNDLE]
            logger.debug("Using CA bundle file at '%s'.", ca_bundle_file)
            if not os.path.isfile(ca_bundle_file):
                raise ValidationError('REQUESTS_CA_BUNDLE environment variable is specified with an invalid file path')
        else:
            ca_bundle_file = certifi.where()
        http.connection_pool_kw['ca_certs'] = ca_bundle_file
    return http


def get_app_service_plan_from_webapp(cmd, webapp, api_version=None):
    client = web_client_factory(cmd.cli_ctx, api_version=api_version)
    plan = parse_resource_id(webapp.server_farm_id)
    return client.app_service_plans.get(plan['resource_group'], plan['name'])


def app_service_plan_exists(cmd, resource_group_name, plan, api_version=None):
    from azure.core.exceptions import ResourceNotFoundError as RNFR
    exists = True
    try:
        client = web_client_factory(cmd.cli_ctx, api_version=api_version)
        client.app_service_plans.get(resource_group_name, plan)
    except RNFR:
        exists = False
    return exists


# Allows putting additional properties on an SDK model instance
def use_additional_properties(resource):
    resource.enable_additional_properties_sending()
    existing_properties = resource.serialize().get("properties")
    resource.additional_properties["properties"] = {} if existing_properties is None else existing_properties


def repo_url_to_name(repo_url):
    repo = None
    repo = [s for s in repo_url.split('/') if s]
    if len(repo) >= 2:
        repo = '/'.join(repo[-2:])
    return repo


def get_token(cmd, repo, token):
    from ._github_oauth import load_github_token_from_cache, get_github_access_token
    if not repo:
        return None
    if token:
        return token
    token = load_github_token_from_cache(cmd, repo)
    if not token:
        token = get_github_access_token(cmd, ["admin:repo_hook", "repo", "workflow"], token)
    return token


def is_logicapp(app):
    if app is None or app.kind is None:
        return False
    return LOGICAPP_KIND in app.kind


def is_functionapp(app):
    if app is None or app.kind is None:
        return False
    return not is_logicapp(app) and FUNCTIONAPP_KIND in app.kind


def is_webapp(app):
    if app is None or app.kind is None:
        return False
    return not is_logicapp(app) and not is_functionapp(app) and "app" in app.kind


def is_linux_webapp(app):
    if not is_webapp(app):
        return False
    return LINUXAPP_KIND in app.kind



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
from typing import TYPE_CHECKING
import warnings

from azure.core.exceptions import ClientAuthenticationError, HttpResponseError, ResourceExistsError, ResourceNotFoundError, map_error
from azure.core.paging import ItemPaged
from azure.core.pipeline import PipelineResponse
from azure.core.pipeline.transport import HttpRequest, HttpResponse
from azure.mgmt.core.exceptions import ARMErrorFormat

from .. import models as _models

if TYPE_CHECKING:
    # pylint: disable=unused-import,ungrouped-imports
    from typing import Any, Callable, Dict, Generic, Iterable, Optional, TypeVar

    T = TypeVar('T')
    ClsType = Optional[Callable[[PipelineResponse[HttpRequest, HttpResponse], T, Dict[str, Any]], Any]]

class AgreementsOperations(object):
    """AgreementsOperations operations.

    You should not instantiate this class directly. Instead, you should create a Client instance that
    instantiates it for you and attaches it as an attribute.

    :ivar models: Alias to model classes used in this operation group.
    :type models: ~azure.mgmt.billing.models
    :param client: Client for service requests.
    :param config: Configuration of service client.
    :param serializer: An object model serializer.
    :param deserializer: An object model deserializer.
    """

    models = _models

    def __init__(self, client, config, serializer, deserializer):
        self._client = client
        self._serialize = serializer
        self._deserialize = deserializer
        self._config = config

    def list_by_billing_account(
        self,
        billing_account_name,  # type: str
        expand=None,  # type: Optional[str]
        **kwargs  # type: Any
    ):
        # type: (...) -> Iterable["_models.AgreementListResult"]
        """Lists the agreements for a billing account.

        :param billing_account_name: The ID that uniquely identifies a billing account.
        :type billing_account_name: str
        :param expand: May be used to expand the participants.
        :type expand: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: An iterator like instance of either AgreementListResult or the result of cls(response)
        :rtype: ~azure.core.paging.ItemPaged[~azure.mgmt.billing.models.AgreementListResult]
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.AgreementListResult"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))
        api_version = "2020-05-01"
        accept = "application/json"

        def prepare_request(next_link=None):
            # Construct headers
            header_parameters = {}  # type: Dict[str, Any]
            header_parameters['Accept'] = self._serialize.header("accept", accept, 'str')

            if not next_link:
                # Construct URL
                url = self.list_by_billing_account.metadata['url']  # type: ignore
                path_format_arguments = {
                    'billingAccountName': self._serialize.url("billing_account_name", billing_account_name, 'str'),
                }
                url = self._client.format_url(url, **path_format_arguments)
                # Construct parameters
                query_parameters = {}  # type: Dict[str, Any]
                query_parameters['api-version'] = self._serialize.query("api_version", api_version, 'str')
                if expand is not None:
                    query_parameters['$expand'] = self._serialize.query("expand", expand, 'str')

                request = self._client.get(url, query_parameters, header_parameters)
            else:
                url = next_link
                query_parameters = {}  # type: Dict[str, Any]
                request = self._client.get(url, query_parameters, header_parameters)
            return request

        def extract_data(pipeline_response):
            deserialized = self._deserialize('AgreementListResult', pipeline_response)
            list_of_elem = deserialized.value
            if cls:
                list_of_elem = cls(list_of_elem)
            return deserialized.next_link or None, iter(list_of_elem)

        def get_next(next_link=None):
            request = prepare_request(next_link)

            pipeline_response = self._client._pipeline.run(request, stream=False, **kwargs)
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, response)
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

            return pipeline_response

        return ItemPaged(
            get_next, extract_data
        )
    list_by_billing_account.metadata = {'url': '/providers/Microsoft.Billing/billingAccounts/{billingAccountName}/agreements'}  # type: ignore

    def get(
        self,
        billing_account_name,  # type: str
        agreement_name,  # type: str
        expand=None,  # type: Optional[str]
        **kwargs  # type: Any
    ):
        # type: (...) -> "_models.Agreement"
        """Gets an agreement by ID.

        :param billing_account_name: The ID that uniquely identifies a billing account.
        :type billing_account_name: str
        :param agreement_name: The ID that uniquely identifies an agreement.
        :type agreement_name: str
        :param expand: May be used to expand the participants.
        :type expand: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: Agreement, or the result of cls(response)
        :rtype: ~azure.mgmt.billing.models.Agreement
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.Agreement"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))
        api_version = "2020-05-01"
        accept = "application/json"

        # Construct URL
        url = self.get.metadata['url']  # type: ignore
        path_format_arguments = {
            'billingAccountName': self._serialize.url("billing_account_name", billing_account_name, 'str'),
            'agreementName': self._serialize.url("agreement_name", agreement_name, 'str'),
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}  # type: Dict[str, Any]
        query_parameters['api-version'] = self._serialize.query("api_version", api_version, 'str')
        if expand is not None:
            query_parameters['$expand'] = self._serialize.query("expand", expand, 'str')

        # Construct headers
        header_parameters = {}  # type: Dict[str, Any]
        header_parameters['Accept'] = self._serialize.header("accept", accept, 'str')

        request = self._client.get(url, query_parameters, header_parameters)
        pipeline_response = self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize('Agreement', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized
    get.metadata = {'url': '/providers/Microsoft.Billing/billingAccounts/{billingAccountName}/agreements/{agreementName}'}  # type: ignore



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
from typing import Any, AsyncIterable, Callable, Dict, Generic, Optional, TypeVar, Union
import warnings

from azure.core.async_paging import AsyncItemPaged, AsyncList
from azure.core.exceptions import ClientAuthenticationError, HttpResponseError, ResourceExistsError, ResourceNotFoundError, map_error
from azure.core.pipeline import PipelineResponse
from azure.core.pipeline.transport import AsyncHttpResponse, HttpRequest
from azure.mgmt.core.exceptions import ARMErrorFormat

from ... import models as _models

T = TypeVar('T')
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, AsyncHttpResponse], T, Dict[str, Any]], Any]]

class RecordSetsOperations:
    """RecordSetsOperations async operations.

    You should not instantiate this class directly. Instead, you should create a Client instance that
    instantiates it for you and attaches it as an attribute.

    :ivar models: Alias to model classes used in this operation group.
    :type models: ~azure.mgmt.privatedns.models
    :param client: Client for service requests.
    :param config: Configuration of service client.
    :param serializer: An object model serializer.
    :param deserializer: An object model deserializer.
    """

    models = _models

    def __init__(self, client, config, serializer, deserializer) -> None:
        self._client = client
        self._serialize = serializer
        self._deserialize = deserializer
        self._config = config

    async def create_or_update(
        self,
        resource_group_name: str,
        private_zone_name: str,
        record_type: Union[str, "_models.RecordType"],
        relative_record_set_name: str,
        parameters: "_models.RecordSet",
        if_match: Optional[str] = None,
        if_none_match: Optional[str] = None,
        **kwargs
    ) -> "_models.RecordSet":
        """Creates or updates a record set within a Private DNS zone.

        :param resource_group_name: The name of the resource group.
        :type resource_group_name: str
        :param private_zone_name: The name of the Private DNS zone (without a terminating dot).
        :type private_zone_name: str
        :param record_type: The type of DNS record in this record set. Record sets of type SOA can be
         updated but not created (they are created when the Private DNS zone is created).
        :type record_type: str or ~azure.mgmt.privatedns.models.RecordType
        :param relative_record_set_name: The name of the record set, relative to the name of the zone.
        :type relative_record_set_name: str
        :param parameters: Parameters supplied to the CreateOrUpdate operation.
        :type parameters: ~azure.mgmt.privatedns.models.RecordSet
        :param if_match: The ETag of the record set. Omit this value to always overwrite the current
         record set. Specify the last-seen ETag value to prevent accidentally overwriting any concurrent
         changes.
        :type if_match: str
        :param if_none_match: Set to '*' to allow a new record set to be created, but to prevent
         updating an existing record set. Other values will be ignored.
        :type if_none_match: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: RecordSet, or the result of cls(response)
        :rtype: ~azure.mgmt.privatedns.models.RecordSet
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.RecordSet"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))
        api_version = "2018-09-01"
        content_type = kwargs.pop("content_type", "application/json")
        accept = "application/json"

        # Construct URL
        url = self.create_or_update.metadata['url']  # type: ignore
        path_format_arguments = {
            'resourceGroupName': self._serialize.url("resource_group_name", resource_group_name, 'str'),
            'privateZoneName': self._serialize.url("private_zone_name", private_zone_name, 'str'),
            'recordType': self._serialize.url("record_type", record_type, 'str'),
            'relativeRecordSetName': self._serialize.url("relative_record_set_name", relative_record_set_name, 'str', skip_quote=True),
            'subscriptionId': self._serialize.url("self._config.subscription_id", self._config.subscription_id, 'str'),
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}  # type: Dict[str, Any]
        query_parameters['api-version'] = self._serialize.query("api_version", api_version, 'str')

        # Construct headers
        header_parameters = {}  # type: Dict[str, Any]
        if if_match is not None:
            header_parameters['If-Match'] = self._serialize.header("if_match", if_match, 'str')
        if if_none_match is not None:
            header_parameters['If-None-Match'] = self._serialize.header("if_none_match", if_none_match, 'str')
        header_parameters['Content-Type'] = self._serialize.header("content_type", content_type, 'str')
        header_parameters['Accept'] = self._serialize.header("accept", accept, 'str')

        body_content_kwargs = {}  # type: Dict[str, Any]
        body_content = self._serialize.body(parameters, 'RecordSet')
        body_content_kwargs['content'] = body_content
        request = self._client.put(url, query_parameters, header_parameters, **body_content_kwargs)
        pipeline_response = await self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200, 201]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        if response.status_code == 200:
            deserialized = self._deserialize('RecordSet', pipeline_response)

        if response.status_code == 201:
            deserialized = self._deserialize('RecordSet', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized
    create_or_update.metadata = {'url': '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Network/privateDnsZones/{privateZoneName}/{recordType}/{relativeRecordSetName}'}  # type: ignore

    async def update(
        self,
        resource_group_name: str,
        private_zone_name: str,
        record_type: Union[str, "_models.RecordType"],
        relative_record_set_name: str,
        parameters: "_models.RecordSet",
        if_match: Optional[str] = None,
        **kwargs
    ) -> "_models.RecordSet":
        """Updates a record set within a Private DNS zone.

        :param resource_group_name: The name of the resource group.
        :type resource_group_name: str
        :param private_zone_name: The name of the Private DNS zone (without a terminating dot).
        :type private_zone_name: str
        :param record_type: The type of DNS record in this record set.
        :type record_type: str or ~azure.mgmt.privatedns.models.RecordType
        :param relative_record_set_name: The name of the record set, relative to the name of the zone.
        :type relative_record_set_name: str
        :param parameters: Parameters supplied to the Update operation.
        :type parameters: ~azure.mgmt.privatedns.models.RecordSet
        :param if_match: The ETag of the record set. Omit this value to always overwrite the current
         record set. Specify the last-seen ETag value to prevent accidentally overwriting concurrent
         changes.
        :type if_match: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: RecordSet, or the result of cls(response)
        :rtype: ~azure.mgmt.privatedns.models.RecordSet
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.RecordSet"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))
        api_version = "2018-09-01"
        content_type = kwargs.pop("content_type", "application/json")
        accept = "application/json"

        # Construct URL
        url = self.update.metadata['url']  # type: ignore
        path_format_arguments = {
            'resourceGroupName': self._serialize.url("resource_group_name", resource_group_name, 'str'),
            'privateZoneName': self._serialize.url("private_zone_name", private_zone_name, 'str'),
            'recordType': self._serialize.url("record_type", record_type, 'str'),
            'relativeRecordSetName': self._serialize.url("relative_record_set_name", relative_record_set_name, 'str', skip_quote=True),
            'subscriptionId': self._serialize.url("self._config.subscription_id", self._config.subscription_id, 'str'),
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}  # type: Dict[str, Any]
        query_parameters['api-version'] = self._serialize.query("api_version", api_version, 'str')

        # Construct headers
        header_parameters = {}  # type: Dict[str, Any]
        if if_match is not None:
            header_parameters['If-Match'] = self._serialize.header("if_match", if_match, 'str')
        header_parameters['Content-Type'] = self._serialize.header("content_type", content_type, 'str')
        header_parameters['Accept'] = self._serialize.header("accept", accept, 'str')

        body_content_kwargs = {}  # type: Dict[str, Any]
        body_content = self._serialize.body(parameters, 'RecordSet')
        body_content_kwargs['content'] = body_content
        request = self._client.patch(url, query_parameters, header_parameters, **body_content_kwargs)
        pipeline_response = await self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        deserialized = self._deserialize('RecordSet', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized
    update.metadata = {'url': '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Network/privateDnsZones/{privateZoneName}/{recordType}/{relativeRecordSetName}'}  # type: ignore

    async def delete(
        self,
        resource_group_name: str,
        private_zone_name: str,
        record_type: Union[str, "_models.RecordType"],
        relative_record_set_name: str,
        if_match: Optional[str] = None,
        **kwargs
    ) -> None:
        """Deletes a record set from a Private DNS zone. This operation cannot be undone.

        :param resource_group_name: The name of the resource group.
        :type resource_group_name: str
        :param private_zone_name: The name of the Private DNS zone (without a terminating dot).
        :type private_zone_name: str
        :param record_type: The type of DNS record in this record set. Record sets of type SOA cannot
         be deleted (they are deleted when the Private DNS zone is deleted).
        :type record_type: str or ~azure.mgmt.privatedns.models.RecordType
        :param relative_record_set_name: The name of the record set, relative to the name of the zone.
        :type relative_record_set_name: str
        :param if_match: The ETag of the record set. Omit this value to always delete the current
         record set. Specify the last-seen ETag value to prevent accidentally deleting any concurrent
         changes.
        :type if_match: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: None, or the result of cls(response)
        :rtype: None
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType[None]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))
        api_version = "2018-09-01"
        accept = "application/json"

        # Construct URL
        url = self.delete.metadata['url']  # type: ignore
        path_format_arguments = {
            'resourceGroupName': self._serialize.url("resource_group_name", resource_group_name, 'str'),
            'privateZoneName': self._serialize.url("private_zone_name", private_zone_name, 'str'),
            'recordType': self._serialize.url("record_type", record_type, 'str'),
            'relativeRecordSetName': self._serialize.url("relative_record_set_name", relative_record_set_name, 'str', skip_quote=True),
            'subscriptionId': self._serialize.url("self._config.subscription_id", self._config.subscription_id, 'str'),
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}  # type: Dict[str, Any]
        query_parameters['api-version'] = self._serialize.query("api_version", api_version, 'str')

        # Construct headers
        header_parameters = {}  # type: Dict[str, Any]
        if if_match is not None:
            header_parameters['If-Match'] = self._serialize.header("if_match", if_match, 'str')
        header_parameters['Accept'] = self._serialize.header("accept", accept, 'str')

        request = self._client.delete(url, query_parameters, header_parameters)
        pipeline_response = await self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200, 204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        if cls:
            return cls(pipeline_response, None, {})

    delete.metadata = {'url': '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Network/privateDnsZones/{privateZoneName}/{recordType}/{relativeRecordSetName}'}  # type: ignore

    async def get(
        self,
        resource_group_name: str,
        private_zone_name: str,
        record_type: Union[str, "_models.RecordType"],
        relative_record_set_name: str,
        **kwargs
    ) -> "_models.RecordSet":
        """Gets a record set.

        :param resource_group_name: The name of the resource group.
        :type resource_group_name: str
        :param private_zone_name: The name of the Private DNS zone (without a terminating dot).
        :type private_zone_name: str
        :param record_type: The type of DNS record in this record set.
        :type record_type: str or ~azure.mgmt.privatedns.models.RecordType
        :param relative_record_set_name: The name of the record set, relative to the name of the zone.
        :type relative_record_set_name: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: RecordSet, or the result of cls(response)
        :rtype: ~azure.mgmt.privatedns.models.RecordSet
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.RecordSet"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))
        api_version = "2018-09-01"
        accept = "application/json"

        # Construct URL
        url = self.get.metadata['url']  # type: ignore
        path_format_arguments = {
            'resourceGroupName': self._serialize.url("resource_group_name", resource_group_name, 'str'),
            'privateZoneName': self._serialize.url("private_zone_name", private_zone_name, 'str'),
            'recordType': self._serialize.url("record_type", record_type, 'str'),
            'relativeRecordSetName': self._serialize.url("relative_record_set_name", relative_record_set_name, 'str', skip_quote=True),
            'subscriptionId': self._serialize.url("self._config.subscription_id", self._config.subscription_id, 'str'),
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}  # type: Dict[str, Any]
        query_parameters['api-version'] = self._serialize.query("api_version", api_version, 'str')

        # Construct headers
        header_parameters = {}  # type: Dict[str, Any]
        header_parameters['Accept'] = self._serialize.header("accept", accept, 'str')

        request = self._client.get(url, query_parameters, header_parameters)
        pipeline_response = await self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        deserialized = self._deserialize('RecordSet', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized
    get.metadata = {'url': '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Network/privateDnsZones/{privateZoneName}/{recordType}/{relativeRecordSetName}'}  # type: ignore

    def list_by_type(
        self,
        resource_group_name: str,
        private_zone_name: str,
        record_type: Union[str, "_models.RecordType"],
        top: Optional[int] = None,
        recordsetnamesuffix: Optional[str] = None,
        **kwargs
    ) -> AsyncIterable["_models.RecordSetListResult"]:
        """Lists the record sets of a specified type in a Private DNS zone.

        :param resource_group_name: The name of the resource group.
        :type resource_group_name: str
        :param private_zone_name: The name of the Private DNS zone (without a terminating dot).
        :type private_zone_name: str
        :param record_type: The type of record sets to enumerate.
        :type record_type: str or ~azure.mgmt.privatedns.models.RecordType
        :param top: The maximum number of record sets to return. If not specified, returns up to 100
         record sets.
        :type top: int
        :param recordsetnamesuffix: The suffix label of the record set name to be used to filter the
         record set enumeration. If this parameter is specified, the returned enumeration will only
         contain records that end with ".:code:`<recordsetnamesuffix>`".
        :type recordsetnamesuffix: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: An iterator like instance of either RecordSetListResult or the result of cls(response)
        :rtype: ~azure.core.async_paging.AsyncItemPaged[~azure.mgmt.privatedns.models.RecordSetListResult]
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.RecordSetListResult"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))
        api_version = "2018-09-01"
        accept = "application/json"

        def prepare_request(next_link=None):
            # Construct headers
            header_parameters = {}  # type: Dict[str, Any]
            header_parameters['Accept'] = self._serialize.header("accept", accept, 'str')

            if not next_link:
                # Construct URL
                url = self.list_by_type.metadata['url']  # type: ignore
                path_format_arguments = {
                    'resourceGroupName': self._serialize.url("resource_group_name", resource_group_name, 'str'),
                    'privateZoneName': self._serialize.url("private_zone_name", private_zone_name, 'str'),
                    'recordType': self._serialize.url("record_type", record_type, 'str'),
                    'subscriptionId': self._serialize.url("self._config.subscription_id", self._config.subscription_id, 'str'),
                }
                url = self._client.format_url(url, **path_format_arguments)
                # Construct parameters
                query_parameters = {}  # type: Dict[str, Any]
                if top is not None:
                    query_parameters['$top'] = self._serialize.query("top", top, 'int')
                if recordsetnamesuffix is not None:
                    query_parameters['$recordsetnamesuffix'] = self._serialize.query("recordsetnamesuffix", recordsetnamesuffix, 'str')
                query_parameters['api-version'] = self._serialize.query("api_version", api_version, 'str')

                request = self._client.get(url, query_parameters, header_parameters)
            else:
                url = next_link
                query_parameters = {}  # type: Dict[str, Any]
                request = self._client.get(url, query_parameters, header_parameters)
            return request

        async def extract_data(pipeline_response):
            deserialized = self._deserialize('RecordSetListResult', pipeline_response)
            list_of_elem = deserialized.value
            if cls:
                list_of_elem = cls(list_of_elem)
            return deserialized.next_link or None, AsyncList(list_of_elem)

        async def get_next(next_link=None):
            request = prepare_request(next_link)

            pipeline_response = await self._client._pipeline.run(request, stream=False, **kwargs)
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                raise HttpResponseError(response=response, error_format=ARMErrorFormat)

            return pipeline_response

        return AsyncItemPaged(
            get_next, extract_data
        )
    list_by_type.metadata = {'url': '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Network/privateDnsZones/{privateZoneName}/{recordType}'}  # type: ignore

    def list(
        self,
        resource_group_name: str,
        private_zone_name: str,
        top: Optional[int] = None,
        recordsetnamesuffix: Optional[str] = None,
        **kwargs
    ) -> AsyncIterable["_models.RecordSetListResult"]:
        """Lists all record sets in a Private DNS zone.

        :param resource_group_name: The name of the resource group.
        :type resource_group_name: str
        :param private_zone_name: The name of the Private DNS zone (without a terminating dot).
        :type private_zone_name: str
        :param top: The maximum number of record sets to return. If not specified, returns up to 100
         record sets.
        :type top: int
        :param recordsetnamesuffix: The suffix label of the record set name to be used to filter the
         record set enumeration. If this parameter is specified, the returned enumeration will only
         contain records that end with ".:code:`<recordsetnamesuffix>`".
        :type recordsetnamesuffix: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: An iterator like instance of either RecordSetListResult or the result of cls(response)
        :rtype: ~azure.core.async_paging.AsyncItemPaged[~azure.mgmt.privatedns.models.RecordSetListResult]
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.RecordSetListResult"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))
        api_version = "2018-09-01"
        accept = "application/json"

        def prepare_request(next_link=None):
            # Construct headers
            header_parameters = {}  # type: Dict[str, Any]
            header_parameters['Accept'] = self._serialize.header("accept", accept, 'str')

            if not next_link:
                # Construct URL
                url = self.list.metadata['url']  # type: ignore
                path_format_arguments = {
                    'resourceGroupName': self._serialize.url("resource_group_name", resource_group_name, 'str'),
                    'privateZoneName': self._serialize.url("private_zone_name", private_zone_name, 'str'),
                    'subscriptionId': self._serialize.url("self._config.subscription_id", self._config.subscription_id, 'str'),
                }
                url = self._client.format_url(url, **path_format_arguments)
                # Construct parameters
                query_parameters = {}  # type: Dict[str, Any]
                if top is not None:
                    query_parameters['$top'] = self._serialize.query("top", top, 'int')
                if recordsetnamesuffix is not None:
                    query_parameters['$recordsetnamesuffix'] = self._serialize.query("recordsetnamesuffix", recordsetnamesuffix, 'str')
                query_parameters['api-version'] = self._serialize.query("api_version", api_version, 'str')

                request = self._client.get(url, query_parameters, header_parameters)
            else:
                url = next_link
                query_parameters = {}  # type: Dict[str, Any]
                request = self._client.get(url, query_parameters, header_parameters)
            return request

        async def extract_data(pipeline_response):
            deserialized = self._deserialize('RecordSetListResult', pipeline_response)
            list_of_elem = deserialized.value
            if cls:
                list_of_elem = cls(list_of_elem)
            return deserialized.next_link or None, AsyncList(list_of_elem)

        async def get_next(next_link=None):
            request = prepare_request(next_link)

            pipeline_response = await self._client._pipeline.run(request, stream=False, **kwargs)
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                raise HttpResponseError(response=response, error_format=ARMErrorFormat)

            return pipeline_response

        return AsyncItemPaged(
            get_next, extract_data
        )
    list.metadata = {'url': '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Network/privateDnsZones/{privateZoneName}/ALL'}  # type: ignore
